local Templates = LibStub:NewLibrary("SomeTemplates-0.3", 1)
if not Templates then return end

local function Copy(templ, source, dest, ...)
  for _,v in ipairs({...}) do
    templ[dest][v] = templ[source][v]
  end
end

local canonical = {}
local aliases = {}

local function makeAliases(templates, name, aliasObj, canonicalObj)
  aliasObj[name] = aliasObj[name] or {}
  local subAlias = aliasObj[name]
  canonicalObj[name] = canonicalObj[name] or {}
  local subCanonical = canonicalObj[name]
  for templateName, template in pairs(templates) do
    subAlias[templateName] = template
    subCanonical[templateName] = templateName
    if template.name then
      subAlias[template.name] = template
      subCanonical[template.name] = templateName
    end
    for also in ipairs(template.also or {}) do
      subAlias[also] = template
      subCanonical[also] = templateName
    end
  end
end

Templates.class = {}

Templates.class.DEATHKNIGHT = {
  [0] = {
    ["Anti-Magic Shell"] = {
      type = "other",
      id = 48707,
      color = "91ff00"
    },
    ["Anti-Magic Zone"] = {
      type = "other",
      id = 51052,
      color = "020047",
      pvp = true
    },
    ["Bone Shield"] = {
      type = "aura",
      id = 195181,
      color = "6d873a"
    },
    ["Corpse Explosion"] = {
      type = "other",
      id = 127344,
      color = "ff0040"
    },
    ["Dark Command"] = {
      type = "other",
      id = 56222,
      color = "ffa769",
      also = {
        ["Murderous Intent"] = "ffa769"
      }
    },
    ["Dark Simulacrum"] = {
      type = "other",
      id = 77606,
      color = "c300c6",
      pvp = true
    },
    ["Death Gate"] = {
      type = "other",
      id = 50977,
      color = "4dff61"
    },
    ["Death Grip"] = {
      type = "other",
      id = 49576,
      color = "e091ff"
    },
    ["Death Strike"] = { h = true,
      id = 49998,
      color = "ff3333",
      display = "aggregate"
    },
    ["Decomposing Aura"] = {
      id = 199720,
      color = "ff3838",
      pvp = true
    },
    ["Frost Breath"] = {
      id = 190780,
      color = "001EFF",
      display = "aggregate"
    },
    ["Icebound Fortitude"] = {
      type = "other",
      id = 48792,
      color = "0a004f"
    },
    ["Mind Freeze"] = {
      type = "other",
      id = 47528,
      color = "FFA8F5"
    },
    ["Raise Ally"] = {
      type = "other",
      id = 61999,
      color = "00ffaa"
    },
    ["Rune of the Fallen Crusader"] = { h = true,
      id = 53344,
      color = "991c03"
    },
    ["Unholy Strength"] = { h = true,
      id = 53365,
      color = "f442e5"
    },
    ["Wraith Walk"] = {
      type = "other",
      id = 212552,
      color = "007287"
    }
  },
  [1] = { -- Blood
    ["Asphyxiate"] = {
      type = "other",
      id = 221562,
      color = "ff5ecc"
    },
    ["Blooddrinker"] = { h = true,
      id = 206931,
      color = "ff5500"
    },
    ["Bloodworms"] = { h = true,
      id = 195679,
      color = "ff26c5",
      display = "aggregate"
    },
    ["Blood Boil"] = { d = true,
      id = 50842,
      color = "ff5454"
    },
    ["Blood for Blood"] = { d = true,
      type = "aura",
      id = 233411,
      color = "ff0707",
      pvp = true
    },
    ["Blood Mirror"] = { d = true,
      id = 206977,
      color = "ff3679"
    },
    ["Blood Plague"] = {
      id = 55078,
      color = "8e1a9e"
    },
    ["Blood Tap"] = {
      type = "other",
      id = 221699,
      color = "ff864a"
    },
    ["Bonestorm"] = { d = true, h = true,
      id = 194844,
      color = "94ff57",
      display = "aggregate"
    },
    ["Consumption"] = { d = true, h = true,
      id = 205223,
      color = "b50033"
    },
    ["Dancing Rune Weapon"] = { d = true, h = true,
      type = "pet",
      name = "Rune Weapon",
      id = 49028,
      color = "ffd000"
    },
    ["Death and Decay"] = { d = true,
      id = 43265,
      color = "c41f1f",
      also = {
        ["Defile"] = "878787"
      }
    },
    ["Death Chain"] = { d = true,
      id = 203173,
      color = "ed004f",
      pvp = true
    },
    ["Death's Caress"] = {
      id = 195292,
      color = "ab0000"
    },
    ["Gorefiend's Grasp"] = {
      id = 108199,
      color = "ac00bf"
    },
    ["Heart Strike"] = { h = true,
      id = 206930,
      color = "ff6969",
      meters = { spell = { "Blood Feast" } }
    },
    ["Icebound Fortitude"] = {
      type = "other",
      id = 48792,
      color = "000a91"
    },
    ["Mark of Blood"] = { h = true,
      id = 206940,
      color = "ed7777"
    },
    ["Marrowrend"] = {
      id = 195182,
      color = "ac00b5"
    },
    ["Rune Tap"] = {
      type = "other",
      id = 194679,
      color = "bd5959"
    },
    ["Soulgorge"] = { d = true,
      id = 212744,
      color = "ff003c",
      display = "aggregate"
    },
    ["Strangulate"] = {
      type = "other",
      id = 47476,
      color = "00ff08",
      pvp = true
    },
    ["Tightening Grasp"] = {
      type = "other",
      id = 206970,
      color = "8300ff"
    },
    ["Tombstone"] = {
      type = "other",
      id = 219809,
      color = "008718"
    },
    ["Vampiric Blood"] = {
      type = "other",
      id = 55233,
      color = "870000"
    }
  },
  [2] = { -- Frost
    ["Avalanche"] = {
      id = 207142,
      color = "1f1a9e",
      display = "individual"
    },
    ["Blinding Sleet"] = {
      type = "other",
      id = 207167,
      color = "4286f4"
    },
    ["Breath of Sindragosa"] = { d = true,
      id = 152279,
      color = "091d3d"
    },
    ["Brittle"] = { d = true,
      id = 214964,
      color = "72a6ff",
      display = "aggregate"
    },
    ["Chill Streak"] = { d = true,
      id = 204160,
      color = "180049",
      pvp = true
    },
    ["Crystalline Swords"] = { d = true,
      id = 189186,
      color = "ce7dc7"
    },
    ["Empower Rune Weapon"] = {
      type = "other",
      id = 47568,
      color = "c6954f",
      also = {
        ["Hungering Rune Weapon"] = "00ddff"
      }
    },
    ["Frost Strike"] = {
      id = 49143,
      color = "1151ff"
    },
    ["Frostscythe"] = {
      id = 207230,
      color = "00ff8c"
    },
    ["Frost Fever"] = {
      id = 55095,
      color = "bad1ff"
    },
    ["Frozen Pulse"] = {
      id = 194909,
      color = "152d56",
      display = "aggregate"
    },
    ["Glacial Advance"] = { d = true,
      id = 194913,
      color = "a3d1ff"
    },
    ["Horn of Winter"] = {
      type = "other",
      id = 57330,
      color = "a3d1ff"
    },
    ["Howling Blast"] = {
      id = 49184,
      color = "0055ff"
    },
    ["Hypothermia"] = { d = true,
      id = 189185,
      color = "564313",
      display = "individual"
    },
    ["Obliterate"] = { d = true,
      id = 49020,
      color = "e4ff3a"
    },
    ["Obliteration"] = { d = true,
      type = "aura",
      id = 207256,
      color = "28deff"
    },
    ["Pillar of Frost"] = { d = true,
      type = "aura",
      id = 51271,
      color = "9fecf9"
    },
    ["Remorseless Winter"] = { d = true,
      id = 196770,
      color = "007a37",
      meters = { spell = { "Frozen Soul" } }
    },
    ["Sindragosa's Fury"] = { d = true,
      id = 190778,
      color = "eeff00",
      meters = { spell = { "Frost Breath" } }
    },
    ["Volatile Shielding"] = { d = true,
      id = 207188,
      color = "0035c6"
    }
  },
  [3] = { --  Unholy
    ["Apocalypse"] = { d = true,
      id = 220143,
      color = "919b00"
    },
    ["Army of the Dead"] = { d = true,
      type = "pet",
      id = 42650,
      color = "cc3fff",
      meters = { spell = { "Army of the Dead" } },
      display = "aggregate"
    },
    ["Blighted Rune Weapon"] = {
      type = "other",
      id = 194918,
      color = "82ffa1"
    },
    ["Bursting Sores"] = { d = true,
      id = 207264,
      color = "1a9e1a"
    },
    ["Chains of Ice"] = {
      type = "other",
      id = 45524,
      color = "8700ff"
    },
    ["Corpse Shield"] = {
      type = "other",
      id = 207319,
      color = "38ff38"
    },
    ["Crypt Fever"] = { d = true,
      id = 199722,
      color = "6dff38",
      pvp = true
    },
    ["Death Coil"] = {
      id = 47541,
      color = "1cff14"
    },
    ["Epidemic"] = {
      id = 207317,
      color = "a5ff44"
    },
    ["Festering Strike"] = {
      id = 85948,
      color = "1814ff"
    },
    ["Festering Wound"] = {
      id = 197147,
      color = "a31b00",
      display = "individual"
    },
    ["Necrotic Strike"] = {
      id = 223829,
      color = "c1553f",
      pvp = true
    },
    ["Outbreak"] = {
      id = 77575,
      color = "8dc13f",
      meters = { spell = { "Pandemic" } }
    },
    ["Raise Dead"] = {
      type = "other",
      id = 46584,
      color = "00ff9d"
    },
    ["Reanimation"] = {
      id = 210128,
      color = "80a53a",
      pvp = true
    },
    ["Scourge Strike"] = {
      id = 55090,
      color = "ff00d8",
      also = {
        ["Clawing Shadows"] = "b43dff"
      }
    },
    ["Soul Reaper"] = {
      id = 130736,
      color = "006caf"
    },
    ["Summon Gargoyle"] = { d = true,
      name = "Ebon Gargoyle",
      type = "pet",
      id = 49206,
      color = "666666",
      also = {
        ["Dark Arbiter"] = "ccf8ff"
      },
      meters = { pet = { "Val'kyr Battlemaiden" } }
    },
    ["The Shambler"] = { d = true,
      type = "pet",
      name = "Shambling Horror",
      id = 191760,
      color = "a5ff00"
    },
    ["Unholy Mutation"] = {
      type = "pet",
      name = "Slime",
      id = 201934,
      color = "00ff6a",
      pvp = true
    },
    ["Virulent Eruption"] = { d = true,
      id = 191685,
      color = "c956b6",
      display = "aggregate"
    },
    ["Virulent Plague"] = {
      id = 191587,
      color = "c956b6"
    },
    ["Wandering Plague"] = {
      id = 199725,
      color = "d8ff00",
      pvp = true
    }
  }
}
Copy(Templates.class.DEATHKNIGHT, 1, 3,
  "Death and Decay"
)

Templates.class.DEMONHUNTER = {
  [0] = {
    ["Blade Dance"] = { d = true,
      id = 188499,
      color = "ff0410",
      also = {
        ["Death Sweep"] = "ff0410"
      }
    },
    ["Consume Magic"] = {
      id = 183752,
      type = "other",
      color = "00F3FF"
    },
    ["Eye of Leotheras"] = { d = true,
      type = "spell",
      id = 206649,
      color = "CDFF00",
      pvp = true
    },
    ["Felblade"] = { d = true,
      id = 232893,
      color = "9AFF00"
    },
    ["Fel Eruption"] = { d = true,
      id = 211881,
      color = "719A00"
    },
    ["Imprison"] = {
      type = "other",
      id = 217832,
      color = "91FF00"
    },
    ["Metamorphosis"] = { d = true,
      id = 191427,
      type = "aura",
      color = "cc9cfc"
    },
    ["Reverse Magic"] = {
      type = "other",
      id = 205604,
      color = "0089FF",
      pvp = true
    },
    ["Soul Fragment"] = { h = true,
      name = "Consume Soul",
      id = 178940,
      color = "cd10fc",
      display = "aggregate"
    },
    ["Spectral Sight"] = {
      type = "other",
      id = 188501,
      color = "A2FF50"
    },
    ["Throw Glaive"] = { d = true,
      id = 185123,
      color = "00FF80"
    }
  },
  [1] = { -- Havoc
    ["Bloodlet"] = {
      id = 206473,
      color = "FF0000"
    },
    ["Blur"] = {
      type = "other",
      id = 198589,
      color = "0044FF",
      also = {
        ["Netherwalk"] = "c300ff"
      }
    },
    ["Chaos Blades"] = { d = true,
      type = "aura",
      id = 211048,
      color = "DEFF00"
    },
    ["Chaos Nova"] = { d = true,
      id = 179057,
      color = "006117"
    },
    ["Chaos Strike"] = {
      id = 162794,
      color = "50FFB1",
      meters = { spell = { "Chaos Cleave" } },
      also = {
        ["Annihilation"] = "50ffb1"
      }
    },
    ["Darkness"] = {
      type = "other",
      id = 196718,
      color = "E600FF"
    },
    ["Demon's Bite"] = {
      id = 162243,
      color = "0243ff",
      disply = "aggregate"
    },
    ["Eye Beam"] = { d = true,
      id = 198013,
      color = "ac01ff",
      meters = { spell = { "Anguish" } }
    },
    ["Fel Barrage"] = { d = true,
      id = 211053,
      color = "00FF0D"
    },
    ["Fel Lance"] = { d = true,
      id = 205313,
      color = "FFEF00",
      also = {
        ["Rain from Above"] = "ffef00"
      },
      pvp = true
    },
    ["Fel Rush"] = { d = true,
      id = 195072,
      override = true,
      color = "008D5E"
    },
    ["Fury of the Illidari"] = { d = true,
      id = 201467,
      color = "fe6400",
      meters = { spell = { "Rage of the Illidari" } }
    },
    ["Inner Demons"] = { d = true,
      id = 201471,
      color = "FF006F"
    },
    ["Mana Break"] = {
      id = 203704,
      color = "0084FF",
      pvp = true
    },
    ["Nemesis"] = {
      type = "other",
      id = 206491,
      color = "FF2600"
    },
    ["Vengeful Retreat"] = { d = true,
      id = 198793,
      color = "fc08dc"
    }
  },
  [2] = { -- Vengeance
    ["Charred Warblades"] = {
      id = 213010,
      color = "c92f00"
    },
    ["Demon Spikes"] = {
      type = "other",
      id = 203720,
      color = "8D8100"
    },
    ["Demonic Trample"] = { d = true,
      id = 205629,
      color = "ffffff",
      pvp = true
    },
    ["Empower Wards"] = {
      type = "other",
      id = 218256,
      color = "3A9700"
    },
    ["Fel Devastation"] = { d = true, h = true,
      id = 212084,
      color = "A5FF99"
    },
    ["Fiery Brand"] = { d = true, h = true,
      id = 204021,
      color = "03fc00",
      meter = { spell = { "Burning Alive" } }
    },
    ["Fracture"] = {
      id = 209795,
      color = "D4FF6E"
    },
    ["Illidan's Grasp"] = { d = true,
      id = 205630,
      color = "FF00EB",
      pvp = true
    },
    ["Immolation Aura"] = { d = true,
      id = 178740,
      color = "FF72E4"
    },
    ["Infernal Strike"] = { d = true,
      id = 189110,
      color = "FF84A3"
    },
    ["Jagged Spikes"] = {
      id = 205627,
      color = "8D8100",
      pvp = true
    },
    ["Nether Bond"] = {
      type = "other",
      id = 207810,
      color = "50007E"
    },
    ["Sever"] = {
      id = 235964,
      color = "01a1fd",
      display = "aggregate"
    },
    ["Shear"] = {
      id = 203782,
      color = "fca807",
      display = "aggregate"
    },
    ["Sigil of Chains"] = {
      type = "other",
      id = 202138,
      color = "FFEF00"
    },
    ["Sigil of Flame"] = { d = true,
      id = 204596,
      color = "f9fc05"
    },
    ["Sigil of Misery"] = {
      type = "other",
      color = "D100FF",
      id = 207684
    },
    ["Sigil of Silence"] = {
      type = "other",
      id = 202137,
      color = "9FEBFF"
    },
    ["Soul Barrier"] = {
      type = "other",
      id = 227225,
      color = "00FF8D"
    },
    ["Soul Carver"] = { d = true,
      id = 207407,
      color = "FFA200"
    },
    ["Soul Cleave"] = { d = true, h = true,
      id = 228477,
      color = "bc01fc"
    },
    ["Spirit Bomb"] = { d = true, h = true,
      id = 218679,
      color = "7300FF"
    },
    ["Torment"] = {
      type = "other",
      id = 185245,
      color = "FF3333",
      also = {
        ["Tormentor"] = "FF3333"
      }
    }
  }
}

Templates.class.DRUID = {
  [0] = {
    ["Cyclone"] = {
      type = "other",
      id = 209753,
      color = "848484",
      also = {
        ["Faerie Swarm"] = "fc55db"
      },
      pvp = true
    },
    ["Dash"] = {
      type = "other",
      id = 1850,
      color = "73FF00"
    },
    ["Displacer Beast"] = {
      type = "other",
      id = 102280,
      color = "55eefc",
      also = {
        ["Wild Charge"] = "55fca9"
      }
    },
    ["Dreamwalk"] = {
      type = "other",
      id = 193753,
      color = "96E0F1"
    },
    ["Ferocious Bite"] = { d = true,
      id = 22568,
      color = "fe003b"
    },
    ["Frenzied Regeneration"] = { h = true,
      id = 22842,
      color = "ff3300"
    },
    ["Growl"] = {
      type = "other",
      id = 6795,
      color = "FF4000",
      also = {
        ["Alpha Challenge"] = "FF4000"
      }
    },
    ["Healing Touch"] = {
      id = 5185,
      color = "01fd14"
    },
    ["Ironfur"] = {
      type = "other",
      id = 192081,
      color = "8FAAB0"
    },
    ["Lunar Strike"] = { d = true,
      id = 194153,
      color = "d101ff"
    },
    ["Mangle"] = {
      id = 33917,
      color = "ff0000",
      also = {
        ["Enraged Mangle"] = "ff0000"
      }
    },
    ["Mighty Bash"] = {
      type = "other",
      id = 5211,
      color = "776D47",
      also = {
        ["Mass Entanglement"] = "406600",
        ["Typhoon"] = "fc37c4"
      }
    },
    ["Moonfire"] = {
      id = 8921,
      color = "5300A6"
    },
    ["Moonkin Form"] = {
      type = "other",
      id = 197625,
      color = "00849B"
    },
    ["Prowl"] = {
      type = "other",
      id = 5215,
      color = "0606B3"
    },
    ["Rake"] = {
      id = 1822,
      color = "FF4E4E"
    },
    ["Rebirth"] = {
      type = "other",
      id = 20484,
      color = "FF4900"
    },
    ["Regrowth"] = {
      id = 8936,
      color = "CCFF63"
    },
    ["Rejuvenation"] = {
      id = 8936,
      color = "FF39EC",
      meters = { spell = { "Revitalize", "Cultivation" } }
    },
    ["Renewal"] = { h = true,
      id = 108238,
      color = "0313dc"
    },
    ["Rip"] = {
      id = 1079,
      color = "9B0000",
      meters = {
        spell = { "Ashamane's Rip" }
      }
    },
    ["Shooting Stars"] = {
      id = 202342,
      color = "7464d1",
      display = "individual"
    },
    ["Shred"] = { d = true, h = true,
      id = 5221,
      color = "ff6104"
    },
    ["Solar Wrath"] = {
      id = 190984,
      color = "fdf901"
    },
    ["Starsurge"] = { d = true,
      id = 78674,
      color = "fe06f4",
      meters = { spell = { "Goldrinn's Fang" } }
    },
    ["Sunfire"] = {
      id = 93402,
      color = "FFD100"
    },
    ["Swiftmend"] = {
      id = 18562,
      color = "5595fe"
    },
    ["Swipe"] = { d = true,
      id = 213764,
      color = "ffb003",
      also = {
        ["Brutal Slash"] = "ff1000"
      }
    },
    ["Ysera's Gift"] = {
      id = 145108,
      color = "00ff72"
    }
  },
  [1] = { -- Balance
    ["Astral Communion"] = {
      --type = "other",
      id = 202359,
      color = "210044"
    },
    ["Barkskin"] = {
      --type = "other",
      id = 22812,
      color = "9F8500"
    },
    ["Blessing of the Ancients"] = {
      type = "other",
      id = 202360,
      color = "73FF00"
    },
    ["Celestial Alignment"] = { d =  true,
      type = "aura",
      id = 194223,
      color = "02ebff",
      also = {
        ["Incarnation: Chosen of Elune"] = "02ebff"
      }
    },
    ["Force of Nature"] = { d = true,
      type = "pet",
      name = "Treant",
      color = "fd6503",
      id = 205636
    },
    ["Fury of Elune"] = { d = true,
      id = 202770,
      color = "7d65ff",
      display = "aggregate"
    },
    ["Innervate"] = {
      type = "other",
      id = 29166,
      color = "000B6E"
    },
    ["Prickling Thorns"] = { d = true,
      id = 200549,
      color = "BEDB00",
      pvp = true
    },
    ["Remove Corruption"] = {
      type = "other",
      id = 2782,
      color = "FFDD7B"
    },
    ["Scythe of Elune"] = { d = true,
      id = 202767,
      color = "06f3fd",
      also = {
        ["New Moon"] = "06f3fd",
        ["Half Moon"] = "06f3fd",
        ["Full Moon"] = "06f3fd"
      }
    },
    ["Shapemender"] = { h = true,
      id = 200308,
      color = "ed4d1c"
    },
    ["Solar Beam"] = {
      type = "other",
      id = 78675,
      color = "F5FF7B"
    },
    ["Starfall"] = { d = true,
      id = 191034,
      color = "0523fe",
      meters = { spell = { "Echoing Stars" } }
    },
    ["Stellar Flare"] = {
      id = 202347,
      color = "7BD1FF"
    },
    ["Touch of the Moon"] = {
      id = 203018,
      color = "00ff26"
    },
    ["Warrior of Elune"] = {
      type = "other",
      id = 202425,
      color = "000286"
    }
  },
  [2] = { -- Feral
    ["Ashamane's Frenzy"] = { d = true,
      id = 210722,
      color = "b78e2d"
    },
    ["Ashamane's Fury"] = { d = true,
      id = 210722,
      color = "FFFC00"
    },
    ["Berserk"] = { d = true,
      type = "aura",
      id = 106951,
      color = "A46A28"
    },
    ["Elune's Guidance"] = {
      type = "other",
      id = 202060,
      color = "210044"
    },
    ["Incarnation: King of the Jungle"] = { d = true,
      type = "aura",
      color = "02ebff",
      id = 102543
    },
    ["Maim"] = {
      id = 22570,
      color = "FF0066"
    },
    ["Rip and Tear"] = {
      type = "other",
      id = 203242,
      color = "8B0000",
      pvp = true
    },
    ["Savage Roar"] = {
      type = "aura",
      id = 52610,
      color = "ffe11f"
    },
    ["Skull Bash"] = {
      type = "other",
      id = 106839,
      color = "ffffff"
    },
    ["Stampeding Roar"] = {
      type = "other",
      id = 77764,
      color = "FFF485"
    },
    ["Survival Instincts"] = {
      type = "other",
      id = 61336,
      color = "622F00"
    },
    ["Thrash"] = {
      id = 106830,
      color = "FFB46E",
      meters = { spell = { "Shadow Thrash" } }
    },
    ["Tiger's Fury"] = {
      type = "aura",
      id = 5217,
      color = "ffb807"
    }
  },
  [3] = { -- Guardian
    ["Blood Claws"] = {
      id = 200815,
      color = "FF2F00"
    },
    ["Brambles"] = {
      id = 203593,
      color = "ff9400",
      display = "aggregate"
    },
    ["Bristling Fur"] = {
      type = "other",
      id = 155835,
      color = "FF7700"
    },
    ["Demoralizing Roar"] = {
      type = "other",
      id = 201664,
      color = "FF7C00",
      pvp = true
    },
    ["Incapacitating Roar"] = {
      type = "other",
      id = 99,
      color = "FF907F"
    },
    ["Incarnation: Guardian of Ursoc"] = { d = true,
      type = "aura",
      id = 102558,
      color = "02ebff",
    },
    ["Lunar Beam"] = { d = true, h = true,
      id = 204066,
      color = "df92ff"
    },
    ["Maul"] = {
      id = 6807,
      color = "568C97"
    },
    ["Mark of Ursol"] = {
      type = "other",
      id = 192083,
      color = "8900FF"
    },
    ["Overrun"] = {
      type = "other",
      id = 202246,
      color = "1A00FF",
      pvp = true
    },
    ["Pulverize"] = { d = true,
      id = 80313,
      color = "4AF1FF"
    },
    ["Rage of the Sleeper"] = { d = true,
      id = 200851,
      color = "FF8D00"
    }
  },
  [4] = { -- Restoration
    ["Cenarion Ward"] = { h = true,
      id = 102351,
      color = "D3FF7F"
    },
    ["Dreamwalker"] = { h = true,
      id = 189849,
      color = "AEFFDB"
    },
    ["Druidic Touch"] = {
      id = 189923,
      color = "F96CFF"
    },
    ["Efflorescence"] = { h = true,
      id = 145205,
      color = "FF005E",
      meters = { spell = { "Spring Blossoms" } }
    },
    ["Essence of G'Hanir"] = { h = true,
      type = "aura",
      id = 208253,
      color = "45A057"
    },
    ["Incarnation: Tree of Life"] = { h = true,
      type = "aura",
      id = 33891,
      color = "02ebff",
    },
    ["Ironbark"] = {
      type = "other",
      id = 102342,
      color = "9D9A59"
    },
    ["Lifebloom"] = {
      id = 33763,
      color = "00FF73",
      meters = { spell = { "Focused Growth" } }
    },
    ["Living Seed"] = { h = true,
      id = 48500,
      color = "37ff00",
      display = "individual"
    },
    ["Nature's Cure"] = {
      type = "other",
      id = 88423,
      color = "898989"
    },
    ["Overgrowth"] = {
      type = "other",
      id = 203651,
      color = "00F3FF",
      pvp = true
    },
    ["Thorns"] = {
      id = 203727,
      color = "907729",
      pvp = true
    },
    ["Tranquility"] = { h = true,
      id = 740,
      color = "5F9ABA"
    },
    ["Ursol's Vortex"] = {
      type = "other",
      id = 102793,
      color = "3F90FF"
    },
    ["Wild Growth"] = { h = true,
      id = 48438,
      color = "84FF9A",
      meters = { spell = { "Dreamwalker", "Nature's Essence" } }
    }
  }
}
Copy(Templates.class.DRUID, 1, 2,
  "Remove Corruption"
)
Copy(Templates.class.DRUID, 1, 3,
  "Barkskin",
  "Remove Corruption"
)
Copy(Templates.class.DRUID, 1, 4,
  "Barkskin",
  "Innervate",
  "Shapemender"
)
Copy(Templates.class.DRUID, 2, 3,
  "Skull Bash",
  "Thrash"
)

Templates.class.HUNTER = {
  [0] = {
    ["A Murder of Crows"] = {
      id = 131894,
      color = "007f74"
    },
    ["Aspect of the Turtle"] = { h = true,
      id = 186265,
      color = "78c900"
    },
    ["Aspect of the Chameleon"] = {
      type = "other",
      id = 61648,
      color = "cecece"
    },
    ["Aspect of the Cheetah"] = {
      type = "other",
      id = 186257,
      color = "ffbf00"
    },
    ["Exhilaration"] = { h = true,
      id = 109304,
      color = "54ff00"
    },
    ["Feed Pet"] = {
      type = "other",
      id = 6991,
      color = "421a00"
    },
    ["Feign Death"] = {
      type = "other",
      id = 5384,
      color = "00e5ff"
    },
    ["Flare"] = {
      type = "other",
      id = 1543,
      color = "ff1500"
    },
    ["Mend Pet"] = {
      type = "other",
      id = 982,
      color = "c8fc58"
    },
    ["Play Dead"] = {
      type = "other",
      id = 209997,
      color = "ff7200"
    },
    ["Scorpid Sting"] = {
      type = "other",
      id = 202900,
      color = "5d00ff",
      pvp =  true
    },
    ["Spider Sting"] = {
      type = "other",
      id = 202914,
      color = "004940",
      pvp = true
    },
    ["Viper Sting"] = {
      type = "other",
      id = 202797,
      color = "8eff8c",
      pvp = true
    },
    ["Wake Up"] = {
      type = "other",
      id = 210000,
      color = "fc8458"
    }
  },
  [1] = { -- Beast Mastery
    ["Aspect of the Beast"] = {
      name = "Bestial Ferocity",
      id = 191384,
      color = "ff5d00"
    },
    ["Aspect of the Wild"] = { d = true,
      type = "aura",
      id = 193530,
      color = "54ff00"
    },
    ["Barrage"] = { d = true,
      id = 120360,
      color = "00ffff"
    },
    ["Bestial Wrath"] = { d = true,
      type = "aura",
      id = 19574,
      color = "ff0000"
    },
    ["Chimaera Shot"] = {
      id = 53209,
      color = "ff2bbb"
    },
    ["Cobra Shot"] = {
      id = 193455,
      color = "1c8c00",
      meters = { spell = { "Wind Arrow" } }
    },
    ["Concussive Shot"] = {
      type = "other",
      id = 5116,
      color = "00128c"
    },
    ["Counter Shot"] = {
      id = 147362,
      color = "757577"
    },
    ["Dire Beast"] = {
      type = "pet",
      name = "Beast",
      id = 120679,
      color = "ffcc00",
      also = {
        ["Dire Frenzy"] = "ff023d"
      },
      meters = { spell = { "Stomp" } }
    },
    ["Dire Beast: Basilisk"] = { d = true,
      type = "pet",
      name = "Basilisk",
      id = 205691,
      color = "ff2a00"
    },
    ["Dire Beast: Hawk"] = { d = true,
      type = "pet",
      name = "Hawk",
      id = 208652,
      color = "fffb87",
      pvp = true
    },
    ["Disengage"] = {
      type = "other",
      id = 781,
      color = "005fdf"
    },
    ["Kill Command"] = {
      id = 34026,
      color = "ff7a42",
      meters = { spell = { "Jaws of Thunder", "Spirit Bond" } }
    },
    ["Misdirection"] = {
      type = "other",
      id = 34477,
      color = "42ffa6"
    },
    ["Multi-Shot"] = {
      id = 2643,
      color = "7a0072",
      display = "aggregate"
    },
    ["Binding Shot"] = {
      type = "other",
      id = 109248,
      color = "fb19ff",
      also = {
        ["Camouflage"] = "0cff00",
        ["Intimidation"] = "969696",
        ["Sticky Bomb"] = "777447",
        ["Wyvern Sting"] = "0cff00"
      }
    },
    ["Stampede"] = { d = true,
      id = 201430,
      color = "ffd800"
    },
    ["Surge of the Stormgod"] = { d = true,
      id = 197354,
      color = "54e2ff",
      display = "aggregate"
    },
    ["Titan's Thunder"] = { d = true,
      id = 207068,
      color = "008cff"
    },
    ["Volley"] = {
      type = "other",
      id = 194386,
      color = "770000"
    }
  },
  [2] = { -- Marksmanship
    ["Aimed Shot"] = {
      id = 19434,
      color = "fcdb76"
    },
    ["Arcane Shot"] = {
      id = 185358,
      color = "a0982b"
    },
    ["Sidewinders"] = { d = true,
      id = 214579,
      color = "a0982b"
    },
    ["Black Arrow"] = {
      id = 194599,
      color = "530087",
      meters = { pet = { "Dark Minion" } }
    },
    ["Bursting Shot"] = { d = true,
      id = 186387,
      color = "ff4800"
    },
    ["Camouflage"] = {
      id = 199483,
      color = "6eff00"
    },
    ["Explosive Shot"] = { d = true,
      id = 212431,
      color = "fff200"
    },
    ["Freezing Arrow"] = {
      type = "other",
      id = 209789,
      color = "002e70"
    },
    ["Marked Shot"] = { d = true,
      id = 185901,
      color = "ff7200",
      meters = { spell = { "Call of the Hunter" } }
    },
    ["Piercing Shot"] = { d = true,
      id = 198670,
      color = "3d60ff"
    },
    ["Scatter Shot"] = {
      id = 213691,
      color = "a5a5a5",
      pvp = true
    },
    ["Sentinel"] = {
      type = "other",
      id = 206817,
      color = "ffffff"
    },
    ["Sniper Shot"] = {
      id = 203155,
      color = "a5410e",
      pvp = true
    },
    ["Trueshot"] = { d = true,
      type = "aura",
      id = 193526,
      color = "fff98c"
    },
    ["Windburst"] = { d = true,
      id = 204147,
      color = "874500"
    }
  },
  [3] = { -- Survival
    ["Carve"] = {
      id = 187708,
      color = "00a9ff",
      also = {
        ["Butchery"] = "a53918"
      }
    },
    ["Dragonsfire Grenade"] = { d = true,
      id = 194855,
      color = "ff3700"
    },
    ["Eagle's Bite"] = {
      name = "On the Trail",
      id = 203757,
      color = "ffae00"
    },
    ["Explosive Trap"] = { d = true,
      id = 191433,
      color = "ff6100"
    },
    ["Flanking Strike"] = {
      id = 202800,
      color = "006aff"
    },
    ["Fury of the Eagle"] = { d = true,
      id = 203415,
      color = "203415"
    },
    ["Harpoon"] = {
      type = "other",
      id = 190925,
      color = "840a0a"
    },
    ["Hatchet Toss"] = {
      id = 193265,
      color = "840a0a"
    },
    ["Lacerate"] = {
      id = 185855,
      color = "7f1010"
    },
    ["Master's Call"] = {
      type = "other",
      id = 53271,
      color = "d3b265"
    },
    ["Mending Bandage"] = { h = true,
      id = 212640,
      color = "b2ff00"
    },
    ["Mongoose Bite"] = { d = true,
      id = 190928,
      color = "ff2323"
    },
    ["Muzzle"] = {
      type = "other",
      id = 187707,
      color = "89008c"
    },
    ["Raptor Strike"] = {
      id = 186270,
      color = "a83c3c",
      display = "aggregate"
    },
    ["Serpent Sting"] = {
      id = 87935,
      color = "fff200"
    },
    ["Snake Hunter"] = {
      type = "other",
      id = 201078,
      color = "1f3447"
    },
    ["Spitting Cobra"] = {
      type = "pet",
      id = 194407,
      color = "ff00f6"
    },
    ["Steel Trap"] = {
      id = 162488,
      color = "f7975b",
      also = {
        ["Freezing Trap"] = "8a5bf7"
      }
    },
    ["Caltrops"] = {
      id = 194277,
      color = "2d2d2d",
      also = {
        ["Tar Trap"] = "2d2d2d"
      }
    },
    ["Throwing Axes"] = {
      id = 200163,
      color = "0026ff"
    },
    ["Tracker's Net"] = {
      id = 212638,
      color = "0026ff",
      pvp = true
    }
  }
}
Copy(Templates.class.HUNTER, 1, 2,
  "Barrage",
  "Binding Shot",
  "Cobra Shot",
  "Concussive Shot",
  "Counter Shot",
  "Disengage",
  "Misdirection",
  "Multi-Shot"
)
Copy(Templates.class.HUNTER, 1, 3,
  "Aspect of the Beast"
)
Copy(Templates.class.HUNTER, 2, 3,
  "Camouflage"
)

Templates.class.MAGE = {
  [0] = {
    ["Blink"] = { h = true,
      id = 1953,
      color = "8700ff",
      also = {
        ["Shimmer"] = "8700ff"
      }
    },
    ["Conjure Refreshment"] = {
      type = "other",
      id = 190336,
      color ="ffbf00"
    },
    ["Counterspell"] = {
      type = "other",
      id = 2139,
      color = "b26bff"
    },
    ["Frost Nova"] = { d = true,
      id = 122,
      color = "0a80ff"
    },
    ["Ice Barrier"] = {
      type = "other",
      id = 11426,
      color = "004d60"
    },
    ["Ice Block"] = {
      type = "other",
      id = 45438,
      color = "000e60"
    },
    ["Illusion"] = {
      type = "other",
      id = 131784,
      color = "ffffff"
    },
    ["Invisibility"] = {
      type = "other",
      id = 66,
      color = "56d2ff"
    },
    ["Ring of Frost"] = {
      type = "other",
      id = 113724,
      color = "000572"
    },
    ["Rune of Power"] = {
      type = "other",
      id = 116011,
      color = "000121"
    },
    ["Temporal Shield"] = { h = true,
      id = 198111,
      color = "a5d9ff"
    },
    ["Time Warp"] = {
      type = "other",
      id = 80353,
      color = "2e0060"
    },
    ["Water Jet"] = {
      id = 135029,
      color = "82fff2"
    }
  },
  [1] = { -- Arcane
    ["Aegwynn's Ascendance"] = { d = true,
      id = 187680,
      color = "3590ff"
    },
    ["Arcane Barrage"] = { d = true,
      id = 44425,
      color = "0026ff",
      meters = { spell = { "Arcane Rebound" } }
    },
    ["Arcane Blast"] = {
      id = 30451,
      color = "20006b",
      display = "aggregate",
      meters = { spell = { "Unstable Magic" } }
    },
    ["Arcane Explosion"] = { d = true,
      id = 1449,
      color = "8a7fd1",
      display = "aggregate"
    },
    ["Arcane Familiar"] = {
      type = "other",
      id = 205022,
      color = "8700ff"
    },
    ["Arcane Missiles"] = { d = true,
      id = 5143,
      color = "5f00b2",
      display = "aggregate"
    },
    ["Arcane Orb"] = { d = true,
      id = 153626,
      color = "a13dff"
    },
    ["Arcane Power"] = { d = true,
      type = "aura",
      id = 12042,
      color = "003bff"
    },
    ["Charged Up"] = {
      type = "other",
      id = 205032,
      color = "110077"
    },
    ["Displacement"] = {
      type = "other",
      id = 195676,
      color = "000b30"
    },
    ["Erosion"] = {
      type = "other",
      id = 205039,
      color = "ff5400"
    },
    ["Evocation"] = {
      type = "other",
      id = 12051,
      color = "ffae7c"
    },
    ["Greater Invisibility"] = {
      type = "other",
      id = 110959,
      color = "21004c"
    },
    ["Mark of Aluneth"] = { d = true,
      id = 224986,
      color = "ff00c7"
    },
    ["Mass Invisibility"] = { d = true,
      id = 198158,
      color = "ff00c7",
      pvp = true
    },
    ["Mirror Image"] = { d = true,
      type = "pet",
      name = "%n",
      id = 55342,
      color = "5ebcff",
      meters = { pet = { "Mirror Image" } }
    },
    ["Nether Tempest"] = {
      id = 114923,
      color = "ff63dd"
    },
    ["Rewind Time"] = {
      type = "other",
      id = 213220,
      color = "ffec5e",
      pvp = true
    },
    ["Supernova"] = { d = true,
      id = 157980,
      color = "7288e0"
    },
    ["Touch of the Magi"] = { d = true,
      id = 210824,
      color = "2705da"
    }
  },
  [2] = { -- Fire
    ["Blast Furnace"] = {
      id = 194487,
      color = "ff2600"
    },
    ["Blast Wave"] = { d = true,
      id = 157981,
      color = "d4ff00"
    },
    ["Cinderstorm"] = { d = true,
      id = 198929,
      color = "6d1d00"
    },
    ["Combustion"] = { d = true,
      type = "aura",
      id = 190319,
      color = "725400"
    },
    ["Conflagration"] = {
      id = 205023,
      color = "722a00",
      meters = { spell = { "Conflagration Flare Up" } }
    },
    ["Dragon's Breath"] = { d = true,
      id = 31661,
      color = "720000"
    },
    ["Fire Blast"] = {
      id = 108853,
      color = "ffac47"
    },
    ["Fireball"] = {
      id = 133,
      color = "d65708",
      meters = { spell = { "Unstable Magic" } }
    },
    ["Flame On"]=  {
      type = "other",
      id = 205029,
      color = "962a00"
    },
    ["Flame Patch"] = {
      id = 205037,
      color = "ff9000"
    },
    ["Flamestrike"] = {
      id = 2120,
      color = "ff6a00",
      meters = { spell = { "Aftershocks", "Flame Patch" } },
      display = "aggregate"
    },
    ["Greater Pyroblast"] = {
      id = 203286,
      color = "ff3700",
      pvp = true
    },
    ["Living Bomb"] = { d = true,
      id = 44457,
      color = "ffd000"
    },
    ["Ignite"] = {
      id = 12846,
      color = "db3e00"
    },
    ["Meteor"] = { d = true,
      id = 153561,
      color = "ce5200",
      meters = { spell = { "Meteor Burn" } }
    },
    ["Phoenix Reborn"] = {
      id = 215773,
      color = "ceb900"
    },
    ["Phoenix's Flames"] = { d = true,
      id = 194466,
      color = "ff9d00"
    },
    ["Pyroblast"] = {
      id = 11366,
      color = "ffa042"
    },
    ["Scorch"] = {
      id = 2948,
      color = "ffcc00"
    }
  },
  [3] = { -- Frost
    ["Blizzard"] = {
      id = 190356,
      color = "273284"
    },
    ["Chilled to the Bone"] = { d = true,
      id = 198126,
      color = "2c3575",
      pvp = true
    },
    ["Comet Storm"] = { d = true,
      id = 153595,
      color = "3149f9"
    },
    ["Cone of Cold"] = { d = true,
      id = 120,
      color = "31adf9"
    },
    ["Ebonbolt"] = { d = true,
      id = 214634,
      color = "5bc0ff"
    },
    ["Flurry"] = {
      id = 44614,
      color = "0600ba",
      display = "aggregate"
    },
    ["Frost Bomb"] = { d = true,
      id = 112948,
      color = "8ef3ff"
    },
    ["Frostbolt"] = {
      id = 116,
      color = "030072",
      meters = { spell = { "Unstable Magic" } }
    },
    ["Frozen Orb"] = { d = true,
      id = 84714,
      color = "0099ff",
      meters = { spell = { "Frost Nova" } }
    },
    ["Frozen Touch"] = {
      type = "other",
      id = 205030,
      color = "003a60"
    },
    ["Glacial Spike"] = { d = true,
      id = 199786,
      color = "2dabff"
    },
    ["Ice Floes"] = {
      id = 108839,
      color = "004577"
    },
    ["Ice Lance"] = { d = true,
      id = 30455,
      color = "62a0f7",
      meters = { spell = { "Icicle" } }
    },
    ["Ice Nova"] = { d = true,
      id = 157997,
      color = "092751"
    },
    ["Icy Veins"] = { d = true,
      type = "aura",
      id = 12472,
      color = "57badb",
      also = {
        ["Ice Form"] = "57badb"
      }
    },
    ["Ray of Frost"] = { d = true,
      id = 205021,
      color = "5ea1ff"
    },
    ["Summon Water Elemental"] = {
      type = "other",
      id = 31687,
      color = "00ff72"
    }
  }
}
Copy(Templates.class.MAGE, 2, 3,
  "Fire Blast"
)

Templates.class.MONK = {
  [0] = {
    ["Chi Burst"] = { d = true,
      id = 123986,
      color = "c001ff"
    },
    ["Crackling Jade Lightning"] = {
      id = 117952,
      color = "7cde9e"
    },
    ["Detox"] = {
      type = "other",
      id = 218164,
      color = "00814E"
    },
    ["Effuse"] = {
      id = 116694,
      color = "00ff9b"
    },
    ["Healing Elixir"] = {
      id = 122281,
      color = "ffd20c",
      also = {
        ["Diffuse Magic"] = "ffd20c",
        ["Dampen Harm"] = "ffd20c"
      }
    },
    ["Leg Sweep"] = {
      type = "other",
      id = 119381,
      color = "ffe368",
      also = {
        ["Ring of Peace"] = "07548c",
        ["Song of Chi-Ji"] = "8c073e",
        ["Summon Black Ox Statue"] = "ffe368"
      }
    },
    ["Paralysis"] = {
      type = "other",
      id = 115078,
      color = "7C0098"
    },
    ["Provoke"] = {
      type = "other",
      id = 115546,
      color = "ffffff",
      also = {
        ["Admonishment"] = "ffffff"
      }
    },
    ["Roll"] = {
      type = "other",
      id = 109132,
      color = "77FF72",
      also = {
        ["Chi Torpedo"] = "77FF72"
      }
    },
    ["Transcendence"] = {
      type = "other",
      id = 101643,
      color = "006B98"
    },
    ["Transcendence: Transfer"] = {
      type = "other",
      id = 119996,
      color = "FF84EB"
    },
    ["Tiger's Lust"] = {
      type = "other",
      id = 116841,
      color = "FFF972"
    },
    ["Tiger Palm"] = {
      id = 100780,
      color = "40bc96"
    },
    ["Zen Pilgrimage"] = {
      type = "other",
      id = 126892,
      color = "006B98"
    }
  },
  [1] = { -- Brewmaster
    ["Blackout Strike"] = {
      id = 205523,
      color = "fa0bff"
    },
    ["Black Ox Brew"] = {
      type = "other",
      id = 115399,
      color = "6BF4FF"
    },
    ["Breath of Fire"] = { d = true,
      id = 115181,
      color = "ff2f0b",
      meters = { spell = { "Dragonfire Brew" } }
    },
    ["Craft: Nimble Brew"] = {
      type = "other",
      id = 213658,
      color = "00A6FF",
      pvp = true
    },
    ["Double Barrel"] = {
      type = "other",
      id = 202335,
      color = "7a5821",
      pvp = true
    },
    ["Elusive Dance"] = { d = true,
      type = "aura",
      id = 196739,
      color = "00ffac"
    },
    ["Expel Harm"] = { d = true, h = true,
      id = 115072,
      color = "00ffac"
    },
    ["Exploding Keg"] = { d = true,
      id = 214326,
      color = "1fff00"
    },
    ["Eye of the Tiger"] = {
      id = 196607,
      color = "ffbb4f"
    },
    ["Fortifying Brew"] = {
      type = "other",
      id = 115203,
      color = "E1BF00"
    },
    ["Guard"] = {
      type = "other",
      id = 202162,
      color = "FFF300",
      pvp = true
    },
    ["Healing Sphere"] = { h = true,
      id = 124502,
      meters = { spell = { "Greater Healing Sphere" } },
      display = "aggregate",
      color = "4AFF35"
    },
    ["Hot Trub"] = { d = true,
      id = 202126,
      color = "493007",
      pvp = true
    },
    ["Incendiary Brew"] = {
      type = "other",
      id = 202272,
      color = "ff0000",
      pvp = true
    },
    ["Invoke Niuzao, the Black Ox"] = { d = true,
      type = "pet",
      name = "Niuzao",
      id = 132578,
      color = "ffc000"
    },
    ["Ironskin Brew"] = {
      type = "other",
      id = 115308,
      color = "57FF71"
    },
    ["Keg Smash"] = { d = true,
      id = 121253,
      color = "ed00ff"
    },
    ["Mighty Ox Kick"] = {
      type = "other",
      id = 202370,
      color = "FFE600",
      pvp = true
    },
    ["Purifying Brew"] = { h = true,
      id = 119582,
      color = "57FF71"
    },
    ["Rushing Jade Wind"] = {
      id = 116847,
      color = "67ff00"
    },
    ["Spear Hand Strike"] = {
      type = "other",
      id = 116705,
      color = "988900"
    },
    ["Special Delivery"] = { d = true,
      id = 196730,
      color = "0055ff",
      display = "aggregate"
    },
    ["Zen Meditation"] = {
      type = "other",
      id = 115176,
      color = "46FF94"
    }
  },
  [2] = { -- Mistweaver
    ["Blackout Kick"] = { h = true,
      id = 100784,
      color = "fa0bff"
    },
    ["Celestial Breath"] = { h = true,
      id = 199641,
      color = "6BFFBD",
      display = "aggregate"
    },
    ["Enveloping Mist"] = {
      id = 124682,
      color = "ffe100"
    },
    ["Essence Font"] = {
      id = 191837,
      color = "00ffef"
    },
    ["Gust of Mists"] = {
      id = 117907,
      color = "85ffcf"
    },
    ["Invoke Chi-Ji, the Red Crane"] = { h = true,
      type = "pet",
      id = 198664,
      color = "ff0000",
      name = "Chi-Ji"
    },
    ["Life Cocoon"] = { h = true,
      id = 116849,
      color = "00ff03"
    },
    ["Mana Tea"] = { h = true,
      type = "aura",
      id = 197908,
      color = "0033ff"
    },
    ["Mistwalk"] = {
      id = 197945,
      color = "74ff00"
    },
    ["Renewing Mist"] = {
      id = 115151,
      color = "00925D"
    },
    ["Revival"] = { h = true,
      id = 115310,
      color = "64F3FF",
      meters = { spell = { "Blessings of Yu'lon" } }
    },
    ["Rising Sun Kick"] = {
      id = 107428,
      color = "ff3a0b"
    },
    ["Refreshing Jade Wind"] = {
      id = 196725,
      color = "00ff68"
    },
    ["Sheilun's Gift"] = { h = true,
      id = 205406,
      color = "ff8d00"
    },
    ["Soothing Mist"] = {
      id = 193884,
      color = "00ff9b"
    },
    ["Spinning Crane Kick"] = { d = true,
      id = 101546,
      color = "2fb800",
      display = "aggregate"
    },
    ["Summon Jade Serpent Statue"] = {
      type = "other",
      id = 115313,
      color = "00FFCD"
    },
    ["The Mists of Sheilun"] = { h = true,
      id = 199887,
      color = "ffffff"
    },
    ["Vivify"] = {
      id = 116670,
      color = "eaff00"
    },
    ["Way of the Crane"] = { d = true, h = true,
      type = "aura",
      id = 216113,
      color = "8B1C00",
      pvp = true
    },
    ["Zen Focus Tea"] = {
      type = "other",
      id = 209584,
      color = "FF8EAC",
      pvp = true
    },
    ["Zen Pulse"] = { d = true, h = true,
      id = 124081,
      color = "ffffff"
    }
  },
  [3] = { -- Windwalker
    ["Chi Orbit"] = { d = true,
      id = 196743,
      color = "01faff"
    },
    ["Chi Wave"] = { d = true, h = true,
      id = 115098,
      color = "41c36c"
    },
    ["Energizing Elixir"] = {
      type = "other",
      id = 115288,
      color = "00FF04"
    },
    ["Fists of Fury"] = { d = true,
      id = 113656,
      override = true,
      color = "204eff",
      meters = { spell = { "Crosswinds" } }
    },
    ["Flying Serpent Kick"] = { d = true,
      id = 101545,
      override = true,
      color = "458f6b"
    },
    ["Fortifying Elixir"] = {
      type = "other",
      id = 201318,
      color = "7a5821",
      pvp = true
    },
    ["Grapple Weapon"] = {
      type = "other",
      id = 233759,
      color = "773C00",
      pvp = true
    },
    ["Invoke Xuen, the White Tiger"] =  { d = true,
      type = "pet",
      name = "Xuen",
      id = 123904,
      color = "81c1ff"
    },
    ["Lightning Burst"] = { d = true,
      id = 224128,
      color = "00E2B1"
    },
    ["Storm, Earth, and Fire"] = { d = true,
      type = "aura",
      id = 137639,
      color = "db59ff",
      also = {
        ["Serenity"] = "75fffa"
      }
    },
    ["Spinning Fire Blossom"] = {
      id = 202077,
      color = "FF5E00",
      pvp = true
    },
    ["Strike of the Windlord"] = { d = true,
      id = 205320,
      override = true,
      color = "ffe400"
    },
    ["Touch of Death"] = { d = true,
      id = 115080,
      color = "ff0000"
    },
    ["Touch of Karma"] = { d = true, h = true,
      id = 122470,
      color = "ffffff"
    },
    ["Whirling Dragon Punch"] = { d = true,
      id = 152175,
      color = "00ff53"
    },
    ["Zen Moment"] = {
      type = "other",
      id = 201325,
      color = "46FF94",
      pvp = true
    }
  }
}
Copy(Templates.class.MONK, 1, 3,
  "Rushing Jade Wind",
  "Healing Sphere",
  "Eye of the Tiger",
  "Spear Hand Strike"
)
Copy(Templates.class.MONK, 2, 3,
  "Blackout Kick",
  "Rising Sun Kick",
  "Spinning Crane Kick"
)

Templates.class.PALADIN = {
  [0] = {
    ["Arcing Light"] = {
      id = 119952,
      color = "ffd49e"
    },
    ["Avenging Wrath"] = { d = true, h = true,
      type = "aura",
      id = 31842,
      color = "ffbb4f",
      also = {
        ["Avenging Crusader"] = "ffbb4f",
        ["Crusade"] = "ffbb4f"
      }
    },
    ["Blessing of Freedom"] = {
      type = "other",
      id = 1044,
      color = "962300"
    },
    ["Blessing of Protection"] = {
      type = "other",
      id = 1022,
      color = "3c3aad",
      also = {
        ["Blessing of Spellwarding"] = "3c3aad"
      }
    },
    ["Blinding Light"] = { d = true,
      id = 115750,
      color = "efb33b"
    },
    ["Consecration"] = {
      --type "dot",
      id = 26573,
      color = "a84e0f"
    },
    ["Crusader Strike"] = {
      id = 35395,
      color = "efdb58",
      also = {
        ["Zeal"] = "efdb58"
      }
    },
    ["Divine Steed"] = {
      type = "other",
      id = 190784,
      color = "003eba"
    },
    ["Divine Shield"] = {
      type = "other",
      id = 642,
      color = "720500"
    },
    ["Flash of Light"] = {
      id = 19750,
      color = "ffffff"
    },
    ["Hammer of Justice"] = {
      type = "other",
      id = 853,
      color = "442587"
    },
    ["Hand of Reckoning"] = {
      type = "other",
      id = 62124,
      color = "707200",
      also = {
        ["Inquisition"] = "707200"
      }
    },
    ["Holy Ritual"] = { h = true,
      id = 199422,
      color = "d16ec5",
      display = "aggregate",
      pvp = true
    },
    ["Judgment"] = {
      id = 20271,
      color = "dd801c",
      meters = { spell = { "Lawbringer" } }
    },
    ["Judgment of Light"] = {
      id = 183778,
      color = "fff64f"
    },
    ["Lay on Hands"] = { h = true,
      id = 633,
      color = "d8d28a"
    },
    ["Luminescence"] = { h = true,
      id = 199428,
      color = "e5b547",
      display = "aggregate",
      pvp = true
    },
    ["Repentance"] = {
      id = 20066,
      color = "4286f4"
    },
    ["Rule of Law"] = {
      type = "other",
      id = 214202,
      color = "30240b"
    }
  },
  [1] = { -- Holy
    ["Aura Mastery"] = {
      type = "other",
      id = 31821,
      color = "d8c84b"
    },
    ["Aura of Mercy"] = { h = true,
      id = 183415,
      color = "916bf9"
    },
    ["Avenging Light"] = { d = true,
      id = 199441,
      color = "c7ff2d",
      pvp = true
    },
    ["Beacon of Faith"] = {
      type = "other",
      id = 156910,
      color = "1de2f4"
    },
    ["Beacon of Light"] = {
      type = "other",
      id = 53563,
      color = "ff2dae",
      also = {
        ["Beacon of Virtue"] = "ff2dae"
      }
    },
    ["Bestow Faith"] = { h = true,
      id = 223306,
      color = "ad8b3a"
    },
    ["Blessing of Sacrifice"] = {
      type = "other",
      id = 6940,
      color = "490c00"
    },
    ["Cleanse"] = {
      type = "other",
      id = 4987,
      color = "f47450"
    },
    ["Divine Favor"] = {
      type = "other",
      id = 210294,
      color = "ba832a"
    },
    ["Divine Protection"] = {
      type = "other",
      id = 498,
      color = "ef9b40"
    },
    ["Holy Avenger"] = { h = true,
      type = "aura",
      id = 105809,
      color = "beff32"
    },
    ["Holy Light"] = {
      id = 82326,
      color = "ffb200"
    },
    ["Holy Prism"] = { d = true, h = true,
      id = 114165,
      color = "fff600"
    },
    ["Holy Shock"] = {
      id = 20473,
      color = "f9af86"
    },
    ["Light of Dawn"] = { h = true,
      id = 85222,
      color = "ffe65e"
    },
    ["Light of the Martyr"] = {
      id = 183998,
      color = "912600"
    },
    ["Light's Hammer"] = { d = true, h = true,
      id = 114158,
      color = "c9a330"
    },
    ["Power of the Silver Hand"] = {
      type = "aura",
      id = 200474,
      color = "47000a"
    },
    ["Tyr's Deliverance"] = { h = true,
      id = 200652,
      color = "cec21a"
    }
  },
  [2] = { -- Protection
    ["Aegis of Light"] = {
      type = "other",
      id = 204150,
      color = "44ffec"
    },
    ["Avenger's Shield"] = { d = true,
      id = 31935,
      color = "bc9403",
      meters = { spell = { "Tyr's Enforcer" } }
    },
    ["Bastion of Light"] = {
      type = "other",
      id = 204035,
      color = "513f00"
    },
    ["Cleanse Toxins"] = {
      type = "other",
      id = 213644,
      color = "00ff55"
    },
    ["Divine Protection"] = {
      id = 498,
      color = "c48736",
      also = {
        ["Ardent Defender"] = "c48736"
      }
    },
    ["Eye of Tyr"] = { d = true,
      id = 209202,
      color = "00fffa"
    },
    ["Guardian of Ancient Kings"] = {
      type = "other",
      id = 86659,
      color = "ba3700",
      also = {
        ["Guardian of the Forgotten Queen"] = "ba3700"
      }
    },
    ["Hammer of the Righteous"] = {
      id = 53595,
      color = "625466",
      also = {
        ["Blessed Hammer"] = "625466"
      }
    },
    ["Light of the Protector"] = { h = true,
      id = 184092,
      color = "fffa00",
      also = {
        ["Hand of the Protector"] = "fffa00"
      }
    },
    ["Judgments of the Pure"] = {
      id = 216860,
      color = "ffff00",
      pvp = true
    },
    ["Rebuke"] = {
      type = "other",
      id = 96231,
      color = "ffdf42"
    },
    ["Seraphim"] = { d = true,
      type = "aura",
      id = 152262,
      color = "03003f"
    },
    ["Shield of the Righteous"] = {
      id = 53600,
      color = "ff00a1"
    },
    ["Shield of Virtue"] = {
      type = "other",
      id = 215652,
      color = "ff51bf",
      pvp = true
    }
  },
  [3] = { -- Retribution
    ["Blade of Justice"] = { d = true,
      id = 184575,
      color = "8e0000",
      also = {
        ["Divine Hammer"] = "8e0000"
      },
      meters = { spell = { "Blade of Light" } }
    },
    ["Blessing of Sanctuary"] = {
      type = "other",
      id = 210256,
      color = "32d6ff",
      pvp = true
    },
    ["Divine Storm"] = { d = true, h = true,
      id = 53385,
      color = "ffe047",
      meters = { spell = {
        "Divine Tempest",
        "Healing Storm"
      } }
    },
    ["Echo of the Highlord"] = {
      id = 186788,
      color = "9e4700",
      display = "aggregate"
    },
    ["Execution Sentence"] = { d = true,
      id = 213757,
      color = "544b00"
    },
    ["Eye for an Eye"] = { d = true,
      id = 205191,
      color = "302f23"
    },
    ["Greater Blessing of Wisdom"] = {
      id = 203539,
      color = "292f6d"
    },
    ["Hammer of Reckoning"] = { d = true,
      id = 204939,
      color = "00ffb6",
      pvp = true
    },
    ["Hand of Hindrance"] = {
      type = "other",
      id = 183218,
      color = "ff00e9"
    },
    ["Holy Wrath"] = { d = true,
      id = 210220,
      color = "ffe900"
    },
    ["Justicar's Vengeance"] = { h = true,
      id = 215661,
      color = "7a0c00"
    },
    ["Retribution"] = { d = true,
      type = "aura",
      id = 183435,
      color = "ff9d00"
    },
    ["Shield of Vengeance"] = { d = true,
      id = 184662,
      color = "878200"
    },
    ["Templar's Verdict"] = {
      id = 85256,
      color = "a3a3a3"
    },
    ["Wake of Ashes"] = { d = true,
      id = 205273,
      color = "ff2600",
      meters = { spell = { "Ashes to Ashes" } }
    },
    ["Word of Glory"] = { h = true,
      id = 210191,
      color = "ffc300"
    }
  }
}
Copy(Templates.class.PALADIN, 1, 2,
  "Blessing of Sacrifice",
  "Divine Protection"
)
Copy(Templates.class.PALADIN, 2, 3,
  "Cleanse Toxins",
  "Rebuke"
)

Templates.class.PRIEST = {
  [0] = {
    ["Fade"] = {
      type = "other",
      id = 586,
      color = "8ECCFF",
      also = {
        ["Greater Fade"] = "8ECCFF"
      }
    },
    ["Mass Dispel"] = {
      type = "other",
      id = 32375,
      color = "5500FF"
    },
    ["Power Infusion"] = { d = true, h = true,
      type = "aura",
      id = 10060,
      color = "ffffff"
    }
  },
  [1] = { -- Discipline
    ["Angelic Feather"] = {
      type = "other",
      id = 121536,
      color = "FFCD00"
    },
    ["Archangel"] = { h = true,
      type = "aura",
      id = 197862,
      color = "FFC900",
      pvp = true
    },
    ["Atonement"] = {
      id = 81749,
      color = "EBFF00"
    },
    ["Clarity of Will"] = {
      type = "other",
      id = 152118,
      color = "FF6600"
    },
    ["Dark Archangel"] = { d = true,
      type = "aura",
      id = 197871,
      color = "AC5DFF",
      pvp = true
    },
    ["Divine Star"] = { d = true, h = true,
      id = 110744,
      color = "FFF990"
    },
    ["Halo"] = { d = true, h = true,
      id = 120517,
      color = "FFF990"
    },
    ["Leap of Faith"] = { h = true,
      type = "other",
      id = 73325,
      color = "9FEFFC"
    },
    ["Light's Wrath"] = { d = true,
      id = 207946,
      color = "990000"
    },
    ["Renewed Hope"] = { h = true,
      id = 197469,
      color = "ce6d63"
    },
    ["Shadowfiend"] = { d = true,
      type = "pet",
      id = 34433,
      color = "fb057a",
      also = {
        ["Mindbender"] = "fb057a"
      }
    },
    ["Shining Force"] = {
      type = "other",
      id = 204263,
      color = "FFF300"
    },
    ["Spirit of the Redeemer"] = { h = true,
      type = "aura",
      id = 215982,
      color = "5AEFFF",
      pvp = true
    },
    ["Pain Suppression"] = {
      type = "other",
      id = 33206,
      color = "B996FF"
    },
    ["Penance"] = { d = true, h = true,
      id = 47540,
      color = "2a2bff",
      meters = { spell = { "N'ero, Band of Promises" } }
    },
    ["Plea"] = {
      id = 200829,
      color = "02d3ff"
    },
    ["Power Word: Barrier"] = {
      type = "other",
      id = 194509,
      color = "FEFFD7"
    },
    ["Power Word: Fortitude"] = {
      type = "other",
      id = 211681,
      color = "63c5ce"
    },
    ["Power Word: Radiance"] = { h = true,
      id = 194509,
      color = "ffbb26",
      also = {
        ["Shadow Covenant"] = "3100af"
      }
    },
    ["Power Word: Shield"] = {
      type = "other",
      id = 17,
      color = "FFB7CE"
    },
    ["Power Word: Solace"] = {
      id = 129250,
      color = "ffdc20"
    },
    ["Premonition"] = { h = true,
      id = 209780,
      color = "FFC900",
      pvp = true
    },
    ["Psychic Scream"] = {
      type = "other",
      id = 8122,
      color = "00C4FF",
      also = {
        ["Mind Bomb"] = "afa400"
      }
    },
    ["Purge the Wicked"] = { d = true,
      id = 204197,
      color = "FF5E00"
    },
    ["Purify"] = {
      type = "other",
      id = 527,
      color = "5B0090"
    },
    ["Rapture"] = {
      type = "other",
      id = 47536,
      color = "7E7E7E"
    },
    ["Ray of Hope"] = { h = true,
      id = 197268,
      color = "FFF885",
      pvp = true
    },
    ["Schism"] = {
      id = 214621,
      color = "ff2df9"
    },
    ["Shadow Mend"] = {
      id = 186263,
      color = "8902ff"
    },
    ["Shadow Word: Pain"] = {
      id = 589,
      color = "FF5E00"
    },
    ["Smite"] = {
      id = 585,
      color = "ffba60"
    }
  },
  [2] = { -- Holy
    ["Apotheosis"] = { h = true,
      type = "aura",
      id = 200183,
      color = "ffba60"
    },
    ["Binding Heal"] = { h = true,
      id = 32546,
      color = "ffe206"
    },
    ["Body and Mind"] = {
      id = 214121,
      color = "ff9569"
    },
    ["Circle of Healing"] = { h = true,
      id = 204883,
      color = "ffba60"
    },
    ["Desperate Prayer"] = { h = true,
      id = 19236,
      color = "ff9569"
    },
    ["Divine Hymn"] = { h = true,
      type = "aura",
      id = 64843,
      idOnly = true,
      color = "ff1b00"
    },
    ["Echo of Light"] = {
      id = 77489,
      color = "ff8720"
    },
    ["Flash Heal"] = {
      id = 2061,
      color = "ffffff",
      meters = { spell = { "Trail of Light" } }
    },
    ["Guardian Spirit"] = { h = true,
      id = 47788,
      color = "32ffff"
    },
    ["Heal"] = {
      id = 2060,
      color = "0261ff"
    },
    ["Holy Fire"] = {
      id = 14914,
      color = "ff8403"
    },
    ["Holy Nova"] = { d = true,
      id = 132157,
      color = "ffc631",
      display = "aggregate"
    },
    ["Holy Ward"] = {
      type = "other",
      id = 213610,
      color = "FFBE5A",
      pvp = true
    },
    ["Holy Word: Chastise"] = {
      id = 88625,
      color = "e0ff84"
    },
    ["Holy Word: Sanctify"] = { h = true,
      id = 34861,
      color = "ffdf00"
    },
    ["Holy Word: Serenity"] = { h = true,
      id = 2050,
      color = "4ddfa4"
    },
    ["Inner Focus"] = {
      type = "other",
      id = 196762,
      color = "ffffff",
      pvp = true
    },
    ["Invoke the Naaru"] = { d = true, h = true,
      id = 196684,
      type = "pet",
      name = "T'uure",
      color = "7c6600"
    },
    ["Light of T'uure"] = { h = true,
      id = 208065,
      color = "000DFF"
    },
    ["Prayer of Healing"] = { h = true,
      id = 596,
      color = "ff7d19"
    },
    ["Prayer of Mending"] = { h = true,
      id = 33076,
      color = "fff802",
      meters = { spell = { "Holy Mending" } }
    },
    ["Renew"] = {
      id = 139,
      color = "00FFC9"
    },
    ["Spirit of Redemption"] = { h = true,
      type = "aura",
      id = 20711,
      color = "b6ffd3",
      also = {
        ["Spirit of the Redeemer"] = "b6ffd3"
      }
    },
    ["Symbol of Hope"] = { h = true,
      type = "aura",
      id = 64901,
      color = "c0ff33"
    }
  },
  [3] = { -- Shadow
    ["Call to the Void"] = {
      type = "pet",
      id = 193470,
      name = "Void Tendril",
      color = "6cff0f"
    },
    ["Dispersion"] = {
      type = "other",
      id = 47585,
      color = "D892FF"
    },
    ["Mind Blast"] = {
      id = 8092,
      color = "ffcce2"
    },
    ["Mind Flay"] = {
      id = 15407,
      color = "02ffda",
      display = "aggregate"
    },
    ["Mind Sear"] = { d = true,
      id = 237388,
      color = "771bff",
      display = "aggregate",
      meters = { spell = { 237388 } }
    },
    ["Psyfiend"] = { d = true,
      id = 211522,
      type = "pet",
      color = "878FFF",
      pvp = true
    },
    ["Purify Disease"] = {
      type = "other",
      id = 213634,
      color = "00FF95"
    },
    ["Shadowy Apparition"] = {
      id = 78203,
      color = "431e9a",
      display = "individual"
    },
    ["Shadow Crash"] = { d = true,
      id = 205385,
      color = "fe41ee"
    },
    ["Shadow Word: Death"] = {
      id = 32379,
      color = "fed702"
    },
    ["Shadow Word: Void"] = {
      id = 205351,
      color = "e9d0fe"
    },
    ["Silence"] = {
      type = "other",
      id = 15487,
      color = "E200FF"
    },
    ["Sphere of Insanity"] = { d = true,
      id = 194179,
      color = "f826fe",
      display = "aggregate"
    },
    ["Surrender to Madness"] = {
      type = "other",
      id = 193223,
      color = "72DFFF"
    },
    ["Vampiric Embrace"] = { h = true,
      id = 15286,
      color = "194dff"
    },
    ["Vampiric Touch"] = {
      id = 34914,
      color = "1500FF"
    },
    ["Voidform"] = { d = true,
      type = "aura",
      id = 228264,
      color = "ff02e3"
    },
    ["Void Bolt"] = {
      id = 228266,
      color = "b00aff"
    },
    ["Void Eruption"] = { d = true,
      id = 228260,
      color = "ff00e2"
    },
    ["Void Shield"] = { h = true,
      id = 199144,
      color = "9A46FF",
      pvp = true
    },
    ["Void Shift"] = {
      type = "other",
      id = 108968,
      color = "C400FF",
      pvp = true
    },
    ["Void Torrent"] = { d = true,
      id = 205065,
      color = "9315ff"
    }
  }
}
Copy(Templates.class.PRIEST, 1, 2,
  "Divine Star",
  "Halo",
  "Leap of Faith",
  "Purify",
  "Shining Force",
  "Smite"
)
Copy(Templates.class.PRIEST, 1, 3,
  "Power Word: Shield",
  "Psychic Scream",
  "Shadowfiend",
  "Shadow Mend",
  "Shadow Word: Pain"
)

Templates.class.ROGUE = {
  [0] = {
    ["Cloak of Shadows"] = {
      type = "other",
      id = 31224,
      color = "3a007c"
    },
    ["Crimson Vial"] = { h = true,
      id = 185311,
      color = "630009"
    },
    ["Distract"] = {
      type = "other",
      id = 1725,
      color = "a01895"
    },
    ["Feint"] = { h = true,
      id = 1966,
      color = "74abff"
    },
    ["Tricks of the Trade"] = {
      type = "other",
      id = 57934,
      color = "545454"
    },
    ["Kick"] = {
      type = "other",
      id = 1766,
      color = "825f31"
    },
    ["Marked for Death"] = {
      type = "other",
      id = 137619,
      color = "d83122"
    },
    ["Sprint"] = {
      type = "other",
      id = 2983,
      color = "ff9000"
    },
    ["Stealth"] = {
      type = "other",
      id = 1784,
      color = "895005"
    },
    ["Vanish"] = {
      type = "other",
      id = 1856,
      color = "a3a3a3"
    }
  },
  [1] = { -- Assassination
    ["Creeping Venom"] = {
      id = 198092,
      color = "006308",
      pvp = true
    },
    ["Poison"] = {
      id = 2823,
      color = "00ff15",
      meters = { spell = {
        "Deady Poison",
        "Wound Poison"
      } }
    },
    ["Envenom"] = { d = true,
      id = 32645,
      color = "4aa018",
      meters = { spell = {
        "System Shock",
        "Poison Bomb"
      } }
    },
    ["Evasion"] = {
      type = "other",
      id = 5277,
      color = "ce5a92"
    },
    ["Eviscerate"] = { d = true,
      id = 196819,
      color = "9b0000"
    },
    ["Exsanguinate"] = {
      type = "other",
      id = 200806,
      color = "ffffff"
    },
    ["Fan of Knives"] = { d = true,
      id = 51723,
      color = "9b0048",
      display = "aggregate",
      meters = { spell = { "Poison Knives" } }
    },
    ["From the Shadows"] = {
      id = 192428,
      color = "bfff00"
    },
    ["Garrote"] = {
      id = 703,
      color = "591818"
    },
    ["Hemorrhage"] = {
      id = 16511,
      color = "ff0000"
    },
    ["Internal Bleeding"] = {
      id = 154904,
      color = "ff3200"
    },
    ["Kidney Shot"] = {
      type = "other",
      id = 408,
      color = "ff3200"
    },
    ["Kingsbane"] = { d = true,
      id = 192759,
      color = "00e9ff"
    },
    ["Poisoned Knife"] = {
      id = 185565,
      color = "3bff00"
    },
    ["Rupture"] = {
      id = 1943,
      color = "910013"
    },
    ["Shadowstep"] = {
      type = "other",
      id = 36554,
      color = "ff0066"
    },
    ["Shiv"] = {
      id = 206328,
      color = "7cff75",
      pvp = true
    },
    ["Sinister Strike"] = {
      id = 1752,
      color = "b600ff",
      also = {
        ["Mutilate"] = "0082c4"
      }
    }
  },
  [2] = { -- Outlaw
    ["Adrenaline Rush"] = { d = true,
      type = "aura",
      id = 13750,
      color = "ffbb63"
    },
    ["Ambush"] = {
      id = 8676,
      color = "a85f00"
    },
    ["Between the Eyes"] = { d = true,
      id = 199804,
      color = "3f3221"
    },
    ["Blade Flurry"] = {
      type = "other",
      id = 13877,
      color = "66270f",
      meters = { spell = { "Shivarran Symmetry" } }
    },
    ["Blind"] = {
      type = "other",
      id = 2094,
      color = "c1a178",
      also = {
        ["Parley"] = "c1a178"
      }
    },
    ["Bribe"] = {
      type = "other",
      id = 199740,
      color = "ffcc00"
    },
    ["Cannonball Barrage"] = { d = true,
      id = 185767,
      color = "ff4300"
    },
    ["Curse of the Dreadblades"] = { d = true,
      type = "aura",
      id = 202665,
      color = "00d0ff"
    },
    ["Dismantle"] = {
      type = "other",
      id = 207777,
      color = "cec456",
      pvp = true
    },
    ["Ghostly Shell"] = { h = true,
      id = 202533,
      color = "3a007c"
    },
    ["Ghostly Strike"] = {
      id = 196937,
      color = "ff3a44"
    },
    ["Gouge"] = {
      type = "other",
      id = 1776,
      color = "af0000"
    },
    ["Grappling Hook"] = {
      type = "other",
      id = 195457,
      color = "4c0000"
    },
    ["Killing Spree"] = { d = true,
      id = 51690,
      color = "ff9000"
    },
    ["Main Gauche"] = {
      id = 86392,
      color = "999999"
    },
    ["Pistol Shot"] = {
      id = 185763,
      color = "7c1a1a",
      meters = { spell = { "Blunderbuss" } }
    },
    ["Plunder Armor"] = { d = true,
      type = "aura",
      id = 198529,
      color = "b7b7b7"
    },
    ["Riposte"] = {
      type = "aura",
      id = 199754,
      color = "00087c"
    },
    ["Run Through"] = { d = true, h = true,
      id = 2098,
      color = "ffee00",
      meters = { spell = { "Greed" } }
    },
    ["Saber Slash"] = {
      id = 193315,
      color = "00f6ff"
    }
  },
  [3] = { -- Subtlety
    ["Backstab"] = {
      id = 53,
      color = "ff2e00",
      also = {
        ["Gloomblade"] = "bb00ff"
      }
    },
    ["Cold Blood"] = {
      id = 213981,
      color = "00ffcb",
      pvp = true
    },
    ["Eviscerate"] = {
      id = 196819,
      color = "8e0000"
    },
    ["Goremaw's Bite"] = { d = true,
      id = 209782,
      color = "6b008e"
    },
    ["Nightblade"] = {
      id = 195452,
      color = "dc00ff"
    },
    ["Shadowstrike"] = {
      id = 185438,
      color = "d3d3d3",
      meters = { spell = { "Dagger in the Dark" } }
    },
    ["Shadowy Duel"] = { d = true,
      type = "aura",
      id = 207736,
      color = "9500af",
      pvp = true
    },
    ["Shadow Blade"] = { d = true,
      id = 121471,
      color = "dc00ff"
    },
    ["Shadow Dance"] = { d = true,
      id = 185313,
      color = "00d8ff"
    },
    ["Shadow Nova"] = { d = true,
      id = 209781,
      color = "9d00ff"
    },
    ["Shuriken Storm"] = { d = true,
      id = 197835,
      color = "00aeff",
      meters = { spell = { "Second Shuriken" } },
      display = "aggregate"
    },
    ["Shuriken Toss"] = {
      id = 114014,
      color = "78a503"
    },
    ["Soul Rip"] = {
      id = 220893,
      color = "fc1bff"
    },
    ["Smoke Bomb"] = {
      type = "other",
      id = 212182,
      color = "daf2d0",
      pvp = true
    },
    ["Symbols of Death"] = {
      type = "aura",
      id = 212283,
      color = "ff00d0"
    }
  }
}
Copy(Templates.class.ROGUE, 1, 3,
  "Evasion",
  "Kidney Shot",
  "Shadowstep"
)
Copy(Templates.class.ROGUE, 2, 3,
  "Blind"
)

Templates.class.SHAMAN = {
  [0] = {
    ["Ascendance"] = { d = true,
      type = "aura",
      id = 114051,
      color = "ff00d9"
    },
    ["Astral Recall"] = {
      type = "other",
      id = 556,
      color = "f45fd9"
    },
    ["Astral Shift"] = {
      type = "other",
      id = 108271,
      color = "d256ff",
      also = {
        ["Ethereal Form"] = "d256ff"
      }
    },
    ["Counterstrike Totem"] = { d = true,
      type = "pet",
      id = 204331,
      color = "0c00ff",
      pvp = true
    },
    ["Earthgrab Totem"] = {
      type = "other",
      id = 2484,
      color = "72432b",
      also = {
        ["Lightning Surge Totem"] = "72432b"
      }
    },
    ["Elemental Healing"] = { h = true,
      id = 198248,
      color = "b200ff"
    },
    ["Ghost Wolf"] = {
      type = "other",
      id = 2645,
      color = "f130ff"
    },
    ["Healing Surge"] = { h = true,
      id = 188070,
      color = "eeff07",
      meters = { spell = { "Swelling Waves" } }
    },
    ["Heroism"] = {
      type = "other",
      id = 2825,
      color = "bc1512",
      also = {
        ["Bloodlust"] = "bc1512"
      }
    },
    ["Hex"] = {
      type = "other",
      id = 51514,
      color = "79797a",
      also = { "Voodoo Totem" }
    },
    ["Lightning Bolt"] = {
      id = 187837,
      color = "0222ff",
      display = "aggregate"
    },
    ["Purge"] = {
      type = "other",
      id = 370,
      color = "c586e0"
    },
    ["Purifying Waters"] = { h = true,
      id = 204247,
      color = "00ff6e",
      pvp = true
    },
    ["Skyfury Totem"] = {
      type = "other",
      id = 204330,
      color = "ff5400",
      pvp = true
    },
    ["Spectral Recovery"] = { h = true,
      id = 204261,
      color = "8d57c6",
      pvp = true
    },
    ["Windfury Totem"] = {
      type = "other",
      id = 204332,
      color = "d979f7",
      pvp = true
    },
    ["Wind Rush Totem"] = {
      type = "other",
      id = 192077,
      color = "6d3110"
    },
    ["Wind Shear"] = {
      type = "other",
      id = 57994,
      color = "6b6b6b"
    }
  },
  [1] = { -- Elemental
    ["Ancestral Guidance"] = { d = true, h = true,
      type = "aura",
      id = 170374,
      color = "0681ff"
    },
    ["Chain Lightning"] = { d = true,
      id = 188443,
      color = "04c8fa",
      also = {
        ["Lava Beam"] = "04c8fa"
      }
    },
    ["Cleanse Spirit"] = {
      type = "other",
      id = 51886,
      color = "7a7a7a"
    },
    ["Earthquake"] = {
      id = 61882,
      color = "cca92a"
    },
    ["Earth Elemental"] = { d = true,
      type = "pet",
      name = "Greater Earth Elemental",
      id = 198103,
      color = "ff830d",
      also = {
        ["Primal Earth Elemental"] = "ff830d"
      }
    },
    ["Earth Shock"] = { d = true,
      id = 8042,
      color = "ffe202"
    },
    ["Elemental Blast"] = {
      id = 117014,
      color = "fa03ff"
    },
    ["Elemental Mastery"] = { d = true,
      type = "aura",
      id = 16166,
      color = "ffc80a"
    },
    ["Fire Elemental"] = { d = true,
      type = "pet",
      name = "Greater Fire Elemental",
      id = 198067,
      color = "ff470b",
      also = {
        ["Storm Elemental"] = "98b4bc",
        ["Primal Fire Elemental"] = "ff470b",
        ["Greater Storm Elemental"] = "98b4bc",
        ["Primal Storm Elemental"] = "98b4bc"
      }
    },
    ["Flame Shock"] = {
      id = 188389,
      color = "ffc700"
    },
    ["Frost Shock"] = {
      id = 196840,
      color = "03ffeb"
    },
    ["Fury of the Storms"] = { d = true,
      type = "pet",
      name = "Greater Lightning Elemental",
      id = 191717,
      color = "001547"
    },
    ["Gust of Wind"] = {
      type = "other",
      id = 192063,
      color = "63daff"
    },
    ["Icefury"] = {
      id = 210714,
      color = "a701ff"
    },
    ["Lava Burst"] = { d = true,
      id = 51505,
      color = "ff0000",
      meters = { spell = {
        "Volcanic Inferno",
        "Pristine Proto-Scale Girdle"
      } },
      display = "aggregate"
    },
    ["Lightning Lasso"] = { d = true,
      id = 204437,
      color = "5aabc4",
      pvp = true
    },
    ["Lightning Rod"] = { d = true,
      id = 197209,
      color = "0050ff",
      display = "aggregate"
    },
    ["Liquid Magma Totem"] = { d = true,
      type = "pet",
      id = 192222,
      color = "ff3501"
    },
    ["Shamanistic Healing"] = { h = true,
      id = 191582,
      color = "005944"
    },
    ["Stormkeeper"] = { d = true,
      type = "aura",
      id = 205495,
      color = "22b592"
    },
    ["Thunderstorm"] = { d = true,
      id = 51490,
      color = "0247fe"
    },
    ["Totem Mastery"] = {
      type = "other",
      id = 210643,
      color = "1900c1"
    }
  },
  [2] = { -- Enhancement
    ["Crash Lightning"] = { d = true,
      id = 187874,
      color = "03a2ff",
      display = "aggregate"
    },
    ["Crashing Storm"] = {
      id = 192246,
      color = "3270ff",
      display = "aggregate"
    },
    ["Doom Vortex"] = { d = true,
      id = 199107,
      color = "ff0000"
    },
    ["Doom Winds"] = { d = true,
      type = "aura",
      id = 204945,
      color = "32fe05"
    },
    ["Earthen Spike"] = { d = true,
      id = 188089,
      color = "ffb800"
    },
    ["Feral Lunge"] = {
      id = 196884,
      color = "04fdeb"
    },
    ["Feral Spirit"] = { d = true,
      type = "pet",
      name = "Spirit Wolf",
      color = "f203fe",
      id = 51533,
      meters = { pet = { "Elemental Raptor" } }
    },
    ["Flametongue"] = {
      id = 193796,
      color = "ff7007"
    },
    ["Frostbrand"] = {
      id = 196834,
      color = "0776ff"
    },
    ["Fury of Air"] = {
      id = 197211,
      color = "fd033d",
      display = "aggregate"
    },
    ["Horrific Slam"] = { d = true,
      id = 222168,
      color = "960014",
      display = "aggregate"
    },
    ["Lava Lash"] = {
      id = 60103,
      color = "0776ff"
    },
    ["Lightning Shield"] = {
      id = 192106,
      color = "4109fd"
    },
    ["Rainfall"] = { h = true,
      id = 215864,
      color = "01b9fd"
    },
    ["Rockbiter"] = {
      id = 193786,
      color = "04d323",
      also = {
        ["Boulderfist"] = "d35304"
      }
    },
    ["Spirit Walk"] = {
      type = "other",
      id = 58875,
      color = "5e0f00"
    },
    ["Stormstrike"] = { d = true,
      id = 17364,
      color = "ffffff",
      meters = { spell = {
        "Forked Lightning",
        "Stormflurry",
        "Unleash Doom",
        "Unleash Lightning",
        "Unleash Lava"
      } }
    },
    ["Sundering"] = { d = true,
      id = 197214,
      color = "fd6407"
    },
    ["Thundercharge"] = {
      type = "other",
      id = 204366,
      color = "00ecfc",
      pvp = true
    },
    ["Windsong"] = {
      id = 201898,
      color = "0bd1ff"
    }
  },
  [3] = { -- Restoration
    ["Chain Heal"] = { h = true,
      id = 1064,
      color = "5801f7"
    },
    ["Cloudburst Totem"] = { h = true,
      type = "pet",
      id = 157153,
      color = "7f441d"
    },
    ["Earthen Shield Totem"] = {
      type = "other",
      id = 198838,
      color = "f847ff",
      also = {
        ["Ancestral Protection Totem"] = "ff4800"
      }
    },
    ["Earth Shield"] = {
      type = "other",
      id = 204288,
      color = "bbff00",
      pvp = true
    },
    ["Electrocute"] = { d = true,
      id = 206642,
      color = "fa75ff",
      pvp = true
    },
    ["Gift of the Queen"] = { h = true,
      id = 207778,
      color = "00fd7e"
    },
    ["Grounding Totem"] = {
      type = "other",
      id = 204336,
      color = "a354ff",
      pvp = true
    },
    ["Healing Tide Totem"] = { h = true,
      type = "pet",
      id = 108280,
      color = "8fdcff"
    },
    ["Healing Rain"] = { h = true,
      id = 37920,
      color = "0077ff"
    },
    ["Healing Stream Totem"] = { h = true,
      type = "pet",
      id = 5394,
      color = "56a5ff",
      meters = { spell = { "Queen's Decree" } }
    },
    ["Healing Wave"] = {
      id = 77472,
      color = "e001f7"
    },
    ["Purify Spirit"] = {
      type = "other",
      id = 77130,
      color = "777777"
    },
    ["Riptide"] = {
      id = 61295,
      color = "00665b",
      meters = {
        spell = { "Rippling Waters" },
        pet = { "Tidal Pools" }
      }
    },
    ["Spiritwalker's Grace"] = {
      type = "other",
      id = 79206,
      color = "aeff00"
    },
    ["Spirit Link Totem"] = {
      type = "other",
      id = 98008,
      color = "5bffa7",
      also = {
        ["Spirit Link"] = "5bffa7"
      }
    },
    ["Wellspring"] = { d = true,
      id = 197995,
      color = "00c187"
    }
  }
}
Copy(Templates.class.SHAMAN, 1, 2,
  "Cleanse Spirit"
)
Copy(Templates.class.SHAMAN, 1, 3,
  "Ancestral Guidance",
  "Chain Lightning",
  "Gust of Wind",
  "Lava Burst"
)

Templates.class.WARLOCK = {
  [0] = {
    ["Casting Circle"] = {
      type = "other",
      id = 221703,
      color = "D482FF",
      pvp = true
    },
    ["Command Demon"] = { d = true,
      id = 119898,
      color = "fff100",
      display = "aggregate",
      also = {
        ["Whiplash"] = "fff100",
        ["Fellash"] = "fff100",
        ["Felstorm"] = "fff100"
      }
    },
    ["Create Soulwell"] = {
      type = "other",
      id = 29893,
      color = "EA58FF"
    },
    ["Curse of Tongues"] = {
      type = "other",
      id = 199890,
      color = "999999",
      also = {
        ["Curse of Weakness"] = "999999",
        ["Curse of Fragility"] = "999999"
      },
      pvp = true
    },
    ["Dark Pact"] = {
      type = "other",
      id = 108416,
      color = "FF4900"
    },
    ["Demonic Gateway"] = {
      type = "other",
      id = 111771,
      color = "A9FF58"
    },
    ["Drain Life"] = { h = true,
      id = 234153,
      color = "10ff0b",
      also = {
        ["Drain Soul"] = "0498d3"
      },
      display = "aggregate"
    },
    ["Grimoire of Service"] = { d = true,
      type = "pet",
      id = 108501,
      color = "ff7c0b",
      override = true,
      meters = { pet = {
        "Imp",
        "Voidwalker",
        "Succubus",
        "Felhunter",
        "Felguard"
      } }
    },
    ["Mortal Coil"] = { h = true,
      id = 6789,
      color = "E953FF",
      also = {
        ["Howl of Terror"] = "E953FF",
        ["Shadowfury"] = "E953FF"
      }
    },
    ["Nether Ward"] = {
      type = "other",
      id = 212295,
      color = "A6FF4C",
      pvp = true
    },
    ["Ritual of Summoning"] = {
      type = "other",
      id = 698,
      color = "7700FF"
    },
    ["Soulstone"] = {
      type = "other",
      id = 20707,
      color = "3C0080"
    },
    ["Soul Harvest"] = { d = true,
      type = "aura",
      id = 196098,
      color = "490bff"
    },
    ["Summon Doomguard"] = { d = true,
      type = "pet",
      name = "Doomguard",
      id = 18540,
      color = "A22F12"
    },
    ["Summon Infernal"] = { d = true,
      type = "pet",
      name = "Infernal",
      id = 1122,
      color = "A8FF92",
      meters = { spell = { "Infernal Awakening" } }
    },
    ["Unending Resolve"] = {
      type = "other",
      id = 104773,
      color = "AF0023"
    }
  },
  [1] = { -- Affliction
    ["Agony"] = {
      id = 980,
      color = "FF2806"
    },
    ["Corruption"] = { h = true,
      id = 172,
      color = "FF0033",
      meters = { spell = { "Harvester of Souls" } }
    },
    ["Grimoire of Sacrifice"] = { d = true,
      name = "Demonic Power",
      id = 108503,
      color = "ff7c0b"
    },
    ["Haunt"] = {
      id = 48181,
      color = "91FF91",
      display = "individual"
    },
    ["Empowered Life Tap"] = {
      type = "aura",
      id = 235157,
      color = "ff0094"
    },
    ["Phantom Singularity"] = { d = true, h = true,
      id = 205179,
      color = "8E59FF"
    },
    ["Reap Souls"] = {
      type = "aura",
      name = "Deadwind Harvester",
      id = 216698,
      color = "ff0000"
    },
    ["Seed of Corruption"] = { d = true,
      id = 27243,
      color = "7700FF"
    },
    ["Siphon Life"] = {
      id = 63106,
      color = "56FF3C"
    },
    ["Soul Flame"] = { d = true,
      id = 199471,
      color = "ff8900",
      override = true
    },
    ["Soul Swap"] = {
      type = "other",
      id = 86121,
      color = "FF00D1",
      pvp = true
    },
    ["Unstable Affliction"] = {
      id = 104773,
      color = "ff0000",
      meters = { spell = { "Compounding Horror" } }
    }
  },
  [2] = { -- Demonology
    ["Call Dreadstalkers"] = {
      type = "pet",
      name = "Dreadstalker",
      id = 104316,
      color = "FF2200"
    },
    ["Call Felhunter"] = {
      type = "other",
      id = 212619,
      color = "FF6B6B",
      also = {
        ["Singe Magic"] = "FF6B6B"
      },
      pvp = true
    },
    ["Call Fel Lord"] = { d = true,
      type = "pet",
      name = "Fel Lord",
      id = 212459,
      color = "C26100",
      pvp = true
    },
    ["Call Observer"] = { d = true,
      type = "pet",
      name = "Observer",
      id = 201996,
      color = "0084FF",
      pvp = true
    },
    ["Demonwrath"] = { d = true,
      id = 193440,
      color = "9e0bff",
      display = "aggregate"
    },
    ["Doom"] = { d = true,
      color = "00a100",
      id = 603,
      display = "aggregate"
    },
    ["Hand of Gul'dan"] = { d = true,
      id = 105174,
      color = "FF2F75"
    },
    ["Implosion"] = { d = true,
      id = 196277,
      color = "FFB700"
    },
    ["Shadowflame"] = { d = true,
      id = 205181,
      color = "FF78B8",
      display = "aggregate"
    },
    ["Shadow Bolt"] = {
      id = 686,
      color = "bb0bff",
      also = {
        ["Demonbolt"] = "bb0bff"
      }
    },
    ["Summon Darkglare"] = { d = true,
      type = "pet",
      name = "Darkglare",
      id = 205180,
      color = "0077ff"
    },
    ["Thal'kiel's Discord"] = { d = true,
      id = 211729,
      override = true,
      color = "860000",
      display = "aggregate"
    },
    ["Thal'kiel's Consumption"] = { d = true,
      id = 211714,
      color = "804DFF"
    }
  },
  [3] = { -- Destruction
    ["Cataclysm"] = { d = true,
      id = 152108,
      color = "FFCD00"
    },
    ["Channel Demonfire"] = { d = true,
      id = 196447,
      color = "DEFF00"
    },
    ["Chaos Bolt"] = { d = true,
      id = 116858,
      color = "0bff0b",
      idOnly = true
    },
    ["Conflagrate"] = {
      id = 17962,
      color = "ff850b"
    },
    ["Dimensional Rift"] = { d = true,
      id = 196586,
      color = "f00bff",
      meters = { pet = {
        "Chaos Tear",
        "Shadowy Tear",
        "Unstable Tear"
      } }
    },
    ["Fel Fissure"] = { d = true,
      id = 200586,
      color = "92ff2d",
      pvp = true
    },
    ["Firestone"] = {
      type = "other",
      id = 212284,
      color = "FF396E",
      pvp = true
    },
    ["Havoc"] = {
      type = "other",
      id = 80240,
      color = "FF0000",
      also = {
        ["Bane of Havoc"] = "FF0000"
      }
    },
    ["Immolate"] = {
      id = 348,
      color = "FF9500"
    },
    ["Incinerate"] = {
      id = 29722,
      color = "ff3c0b"
    },
    ["Rain of Fire"] = { d = true,
      id = 5740,
      color = "F7FF77",
      display = "aggregate"
    },
    ["Shadowburn"] = {
      id = 17877,
      color = "B4CDFF"
    }
  }
}
Copy(Templates.class.WARLOCK, 1, 3,
  "Grimoire of Sacrifice",
  "Mana Tap"
)

Templates.class.WARRIOR = {
  [0] = {
    ["Avatar"] = { d = true,
      type = "aura",
      id = 107574,
      color = "727272"
    },
    ["Battle Cry"] = { d = true,
      type = "aura",
      id = 1719,
      color = "ff4d2d"
    },
    ["Berserker Rage"] = {
      type = "other",
      id = 18499,
      color = "a82301"
    },
    ["Bladestorm"] = { d = true,
      id = 227847,
      color = "ffca2b",
      also = {
        ["Ravager"] = "ffca2b"
      }
    },
    ["Corrupted Blood of Zakajz"] = { d = true,
      id = 209569,
      color = "99020e"
    },
    ["Charge"] = { d = true,
      id = 100,
      color = "ff0000",
      meters = { spell = { "Warbringer" } }
    },
    ["Deep Wounds"] = {
      id = 115767,
      color = "d82929"
    },
    ["Heroic Leap"] = { d = true,
      id = 6544,
      color = "e85555"
    },
    ["Heroic Throw"] = {
      id = 57755,
      color = "00e817"
    },
    ["Pummel"] = {
      type = "other",
      id = 6552,
      color = "b78c2f"
    },
    ["Shockwave"] = { d = true,
      id = 46968,
      color = "2dedff"
    },
    ["Storm Bolt"] = {
      id = 107570,
      color = "00ff87"
    },
    ["Taunt"] = {
      type = "other",
      id = 355,
      color = "ff2600",
      also = {
        ["Oppressor"] = "ff2600"
      }
    }
  },
  [1] = { -- Arms
    ["Cleave"] = {
      id = 845,
      color = "80ff2b",
      display = "aggregate",
      meters = { spell = { "Void Cleave" } }
    },
    ["Colossus Smash"] = {
      id = 167105,
      color = "bf7500",
      meters = { spell = { "Echo Slam" } }
    },
    ["Commanding Shout"] = {
      type = "other",
      id = 97462,
      color = "4f3000"
    },
    ["Corrupted Blood of Zakajz"] = { d = true,
      id = 209566,
      color = "4f0000",
      display = "aggregate"
    },
    ["Defensive Stance"] = {
      type = "other",
      id = 197690,
      color = "0157ad"
    },
    ["Die by the Sword"] = {
      type = "other",
      id = 118038,
      color = "ff893f"
    },
    ["Execute"] = { d = true,
      id = 163201,
      color = "ff3f00"
    },
    ["Focused Rage"] = {
      type = "other",
      id = 207982,
      color = "ff5e4c"
    },
    ["Hamstring"] = {
      id = 1715,
      color = "ffca65"
    },
    ["Intimidating Shout"] = {
      type = "other",
      id = 5246,
      color = "9d00ff"
    },
    ["Mortal Strike"] = { d = true, h = true,
      id = 12294,
      color = "010270",
      meters = { spell = { "Touch of Zakajz" } }
    },
    ["Opportunity Strikes"] = {
      id = 203179,
      color = "700101"
    },
    ["Overpower"] = {
      id = 7384,
      color = "7a7a7a"
    },
    ["Rend"] = {
      id = 772,
      color = "b21111"
    },
    ["Second Wind"] = { h = true,
      id = 29838,
      color = "ff0048",
      display = "aggregate"
    },
    ["Sharpen Blade"] = {
      type = "other",
      id = 198817,
      color = "00a8b7"
    },
    ["Slam"] = {
      id = 1464,
      color = "320051"
    },
    ["Victory Rush"] = { d = true, h = true,
      id = 34428,
      color = "af6124",
      also = {
        ["Impending Victory"] = "af6124"
      }
    },
    ["Warbreaker"] = { d = true,
      id = 209577,
      color = "017573"
    },
    ["Whirlwind"] = { d = true,
      id = 1680,
      color = "47d4ff",
      display = "aggregate"
    }
  },
  [2] = { -- Fury
    ["Battle Trance"] = { h = true,
      id = 213857,
      color = "681a00",
      pvp = true
    },
    ["Bloodbath"] = {
      id = 12292,
      color = "ad2b00"
    },
    ["Bloodthirst"] = { h = true,
      id = 23381,
      color = "c92c1e"
    },
    ["Dragon Roar"] = { d = true,
      id = 118000,
      color = "9b251a"
    },
    ["Enraged Regeneration"] = {
      id = 184364,
      color = "9b341a"
    },
    ["Furious Slash"] = {
      id = 100130,
      color = "9b9b9b"
    },
    ["Intimidating Shout"] = {
      type = "other",
      id = 5246,
      color = "7a02d6"
    },
    ["Odyn's Fury"] = { d = true,
      id = 205545,
      color = "ffae00"
    },
    ["Raging Blow"] = {
      id = 85288,
      color = "424242"
    },
    ["Rampage"] = {
      id = 184367,
      color = "ff2e00"
    },
    ["Rend Flesh"] = {
      id = 221770,
      color = "ff0c2a"
    }
  },
  [3] = { -- Protection
    ["Bodyguard"] = {
      type = "other",
      id = 213871,
      color = "6b1414",
      pvp = true
    },
    ["Demoralizing Shout"] = {
      type = "other",
      id = 1160,
      color = "186002"
    },
    ["Devastate"] = {
      id = 20243,
      color = "0099ff",
      display = "aggregate"
    },
    ["Dragon Charge"] = { d = true,
      id = 206572,
      color = "961d0a"
    },
    ["Focused Rage"] = {
      type = "other",
      id = 204488,
      color = "d83c24"
    },
    ["Ignore Pain"] = {
      type = "other",
      id = 190456,
      color = "ffcc00"
    },
    ["Intercept"] = {
      type = "other",
      id = 198304,
      color = "f2ff00"
    },
    ["Last Stand"] = {
      type = "other",
      id = 12975,
      color = "44311b"
    },
    ["Neltharion's Fury"] = { d = true,
      id = 203524,
      color = "d8d8d8"
    },
    ["Ravager"] = { d = true,
      id = 228920,
      color = "cc321e"
    },
    ["Renewed Fury"] = { d = true,
      type = "aura",
      id = 202288,
      color = "6d0015"
    },
    ["Revenge"] = { d = true,
      id = 6572,
      color = "ce9177"
    },
    ["Scales of Earth"] = { d = true,
      id = 189059,
      color = "8eba4e"
    },
    ["Shield Bash"] = {
      id = 198912,
      color = "4c3f3a",
      pvp = true
    },
    ["Shield Block"] = {
      type = "other",
      id = 2565,
      color = "152084"
    },
    ["Shield Slam"] = {
      id = 23922,
      color = "b58a3b"
    },
    ["Shield Wall"] = {
      type = "other",
      id = 871,
      color = "2c0070"
    },
    ["Spell Reflection"] = {
      type = "other",
      id = 23920,
      color = "f2f4cd",
      also = {
        ["Mass Spell Reflection"] = "f2f4cd"
      }
    },
    ["Thunder Clap"] = { d = true,
      id = 6343,
      color = "00590b"
    },

  }
}
Copy(Templates.class.WARRIOR, 1, 2,
  "Commanding Shout",
  "Execute",
  "Whirlwind"
)
Copy(Templates.class.WARRIOR, 1, 3,
  "Victory Rush"
)

for _,templates in pairs(Templates.class) do
  if templates[0] then for name,info in pairs(templates[0]) do
    for i=1,#templates do
      templates[i][name] = info
    end
  end end
end


aliases.class = {}
canonical.class = {}
for className,class in pairs(Templates.class) do
  aliases.class[className] = {}
  canonical.class[className] = {}
  local classAliases = aliases.class[className]
  local classCanonical = canonical.class[className]
  for specName,spec in pairs(class) do
    makeAliases(spec, specName, classAliases, classCanonical)
  end
end


Templates.all = {
  ["Bloodlust"] = {
    type = "aura",
    id = 2825,
    color = "bc1512",
    also = {
      ["Time Warp"] = "5300ba",
      ["Heroism"] = "bc1512"
    }
  },
  ["Honorable Medallion"] = {
    type = "other",
    id = 208683,
    color = "FFD100",
    also = {
      ["Gladiator's Medallion"] = "FFD100"
    },
    pvp = true
  },
  ["Mark of the Ancient Priestess"] = {
    id = 228410,
    color = "f5c204"
  },
  ["Mark of the Hidden Satyr"] = {
    id = 191259,
    color = "FF6C6C"
  }
}
makeAliases(Templates.all, "all", aliases, canonical)

Templates.race = {}
Templates.race.Draenei = {
  ["Gift of the Naaru"] = {
    id = 121093,
    color = "cc2b00"
  }
}
Templates.race.Dwarf = {
  ["Stoneform"] = {
    type = "other",
    id = 20594,
    color = "ff6f52"
  }
}
Templates.race.Gnome = {
  ["Escape Artist"] = {
    type = "other",
    id = 20589,
    color = "c7b107"
  }
}
Templates.race.Human = {
  ["Every Man for Himself"] = {
    type = "other",
    id = 59752,
    color = "df75c1"
  }
}
Templates.race.NightElf = {
  ["Shadowmeld"] = {
    type = "other",
    id = 58984,
    color = "be6bdf"
  }
}
Templates.race.Worgen = {
  ["Darkflight"] = {
    type = "other",
    id = 68992,
    color = "df93ab"
  }
}
Templates.race.BloodElf = {
  ["Arcane Torrent"] = {
    type = "other",
    id = 28730,
    color = "0400FF"
  },
}
Templates.race.Goblin = {
  ["Pack Hobgoblin"] = {
    type = "other",
    id = 69046,
    color = "ffc328"
  },
  ["Rocket Barrage"] = { d = true,
    id = 69041,
    color = "ff3b48"
  },
  ["Rocket Jump"] = {
    type = "other",
    id = 69070,
    color = "ff6f61"
  }
}
Templates.race.Orc = {
  ["Blood Fury"] = { d = true, h = true,
    type = "aura",
    id = 33697,
    color = "ff994a"
  }
}
Templates.race.Tauren = {
  ["War Stomp"] = {
    type = "other",
    id = 20549,
    color = "ffd76f"
  }
}
Templates.race.Troll = {
  ["Berserking"] = { d = true, h = true,
    type = "aura",
    id = 26297,
    color = "92d2b1"
  }
}
Templates.race.Scourge = {
  ["Cannibalize"] = { h = true,
    id = 20577,
    color = "fe4735"
  },
  ["Touch of the Grave"] = {
    id = 5227,
    color = "59fe40",
    display = "aggregate"
  },
  ["Will of the Forsaken"] = {
    type = "other",
    id = 7744,
    color = "555555"
  }
}
Templates.race.Pandaren = {
  ["Quaking Palm"] = {
    type = "other",
    id = 107079,
    color = "fcbe15"
  }
}
aliases.race = {}
canonical.race = {}
for raceName, race in pairs(Templates.race) do
  makeAliases(race, raceName, aliases.race, canonical.race)
end

Templates.item = {
  ---------------
  -- Consumables
  ---------------
  [127843] = { d = true,
    name = "Potion of Deadly Grace",
    color = "db7400",
    display = "aggregate",
    meters = { spell = { "Deadly Grace" } }
  },
  [142117] = {
    name = "Potion of Prolonged Power",
    type = "aura",
    color = "ffffff"
  },
  [127844] = { d = true,
    name = "Potion of the Old War",
    color = "db7400",
    display = "aggregate"
  },
  [133574] = { d = true,
    name = "Fishbrul Special",
    override = false,
    color = "f54200",
    meters = { spell = { "Pepper Breath" } },
    display = "aggregate"
  },

  ---------------
  -- Legendaries
  ---------------
  [133976] = {
    name = "Cinidaria, the Symbiote",
    color = "ff7500",
    meters = { spell = { "Symbiote Strike" } }
  },
  [137049] = {
    name = "Insignia of Ravenholdt",
    color = "777777",
    meters = { spell = { "Insignia of Ravenholdt" } },
    display = "aggregate"
  },
  [144259] = { d = true,
    name = "Kil'jaeden's Burning Wish",
    color = "ff4c72"
  },
  [137064] = {
    name = "The Shadow Hunter's Voodoo Mask",
    color = "ffdb76"
  },
  [137107] = {
    name = "Mannoroth's Bloodletting Manacles",
    color = "ffdb76",
    display = "aggregate"
  },
  [137103] = {
    name = "Storm Tempests",
    color = "74abff",
    display = "aggregate"
  },
  [132466] = {
    name = "Roots of Shaladrassil",
    color = "74abff",
    meters = { spell = { "Shaladrassil's Nourishment" } },
    display = "aggregate"
  },
  [133974] = {
    name = "Lana'thel's Lament",
    type = "aura",
    color = "c4092a"
  },
  [144258] = { h = true,
    type = "aura",
    name = "Velen's Future Sight",
    color = "feef0c"
  },
  [144249] = { d = true,
    name = "Archimonde's Hatred Reborn",
    color = "a1fe26"
  },
  [132376] = { h = true,
    name = "Acherus Drapes",
    color = "555555",
    meters = { spell = { "Anti-Magic Shell"} }
  },
  [132379] = { d = true,
    type = "aura",
    name = "Sin'dorei Spite",
    color = "FF9100",
    meters = { aura = { "Sin'dorei Spite" } }
  },

  -------------
  -- Nighthold
  -------------
  [140795] = { h = true,
    name = "Aluriel's Mirror",
    color = "9f8541",
    meters = { spell = { "Nightwell Tranquility" } }
  },
  [140794] = { d = true,
    name = "Arcanogolem Digit",
    color = "ff258a",
    meters = { spell = { "Arcane Swipe" } }
  },
  [140790] = { d = true,
    name = "Claw of the Crystalline Scorpid",
    color = "f260ff",
    meters = { spell = { "Crystalline Shockwave" } }
  },
  [140808] = { d = true,
    name = "Draught Of Souls",
    color = "ad2eff",
    meters = { spell = { "Fel-Crazed Rage" } }
  },
  [140801] = { d = true,
    name = "Fury of the Burning Sky",
    color = "ffca1a",
    meters = { spell = { "Solar Collapse"} }
  },
  [140798] = { d = true,
    name = "Icon of Rot",
    color = "bcd030",
    meters = { spell = { "Carrion Swarm" } }
  },
  [140799] = { d = true,
    name = "Might of Krosus",
    color = "a8d051",
    meters = { spell = { "Colossal Slam" } }
  },
  [140802] = {
    name = "Nightblooming Frond",
    color = "d041c2",
    meters = { spell = { "Recursive Strikes" } }
  },
  [140800] = { d = true,
    name = "Pharamere's Forbidden Grimoire",
    color = "d02013",
    meters = { spell = { "Orb of Destruction" } }
  },
  [140804] = { d = true,
    name = "Star Gate",
    color = "20add0",
    meters = { spell = { "Nether Meteor"} }
  },
  ---------------------
  -- Emerald Nightmare
  ---------------------
  [139334] = { d = true,
    name = "Nature's Call",
    color = "18ff00",
    meters = { spell = { "Cleansed Drake's Breath" } }
  },
  [138224] = {
    name = "Unstable Horrorslime",
    meters = { spell = { "Volatile Ichor" } },
    color = "d02528"
  },
  [139336] = {
    name = "Bough of Corruption",
    color = "d070ca",
    type = { spell = { "Nightmare Corruption" } },
    display = "aggregate"
  },
  [139326] = {
    name = "Wriggling Sinew",
    meters = { spell = { "Maddening Whispers" } },
    color = "f509bc",
    display = "aggregate"
  },
  [139325] = {
    name = "Spontaneous Appendages",
    color = "e7f506",
    meters = { spell = { "Horrific Appendages" } }
  },
  [139320] = {
    name = "Ravaged Seed Pod",
    color = "737a03",
    meters = { spell = { "Infested Ground" } }
  },
  [139323] = { d = true,
    name = "Twisting Wind",
    color = "d3fe71",
    meters = { spell = { "Tormenting Cyclone" } }
  },
  [139330] = {
    name = "Heightened Senses",
    type = "aura",
    color = "7b04f6"
  },
  [139329] = {
    name = "Bloodthirsty Instinct",
    type = "aura",
    color = "f67c6b"
  },
  [139333] = { h = true,
    name = "Horn of Cenarius",
    color = "ff8f49",
    meters = { spell = { "Cleansing Wisp" } }
  },
  [139321] = {
    name = "Swarming Plaguehive",
    color = "ff7700",
    meters = { spell = { "Plague Swarm" } },
    display = "aggregate"
  },
  ------------
  -- Dungeons
  ------------
  [137446] = { d = true,
    name = "Elementium Bomb Squirrel Generator",
    meters = { spell = { "Mechanical Bomb Squirrel"} },
    color = "85fba0"
  },
  [137433] = {
    name = "Obelisk of the Void",
    color = "a60704",
    type = "aura",
    meters = { aura = { "Collapsing Shadow" } }
  },
  [137440] = { d = true,
    name = "Shivermaw's Jawbone",
    color = "07d1fb",
    meters = { spell = { "Ice Bomb" } }
  },
  [137306] = {
    name = "Oakheart's Gnarled Root",
    color = "cf0130",
    meters = { spell = { "Nightmare Essence" } }
  },
  [142157] = { d = true,
    name = "Aran's Relaxing Ruby",
    meters = {
      pet = { "Flame Wreath" },
      spell = { "Flame Wreath" }
    },
    color = "ff0000"
  },
  [137439] = { d = true,
    name = "Tiny Oozeling in a Jar",
    color = "90fb00",
    meters = { spell = { "Fetid Regurgitation" } }
  },
  [136716] = { d = true,
    name = "Caged Horror",
    color = "b601fb",
    meters = { spell = { "Dark Blast" } }
  },
  [136713] = {
    name = "Spiked Counterweight",
    color = "a1fb38",
    meters = { spell = { "Brutal Haymaker"} }
  },
  [137301] = {
    name = "Corrupted Starlight",
    color = "fb0c77",
    meters = { spell = { "Nightmare Star" } }
  },
  [137378] = { h = true,
    name = "Bottled Hurricane",
    color = "05d1fb",
    meters = { spell = { "Raging Storm" } }
  },
  [137369] = { d = true,
    name = "Giant Ornamental Pearl",
    color = "2b74a6",
    meters = { spell = { "Gaseous Explosion"} }
  },
  [137362] = {
    name = "Parjesh's Medallion",
    type = "aura",
    color = "0bf8fa",
    meters = { aura = { "Warlord's Fortitude" } }
  },
  [137373] = {
    name = "Tempered Egg of Serpentrix",
    type = "pet",
    color = "b72511",
    meters = { pet = { "Spawn of Serpentrix" } }
  },
  [133641] = { d = true,
    name = "Eye of Skovald",
    color = "c3fe23",
    meters = { spell = { "Fel Meteor" } }
  },
  [133642] = {
    name = "Horn of Valarjar",
    color = "7c8696",
    type = "aura",
    meters = { aura = { "Valarjar's Path" } }
  },

  [137486] = { d = true,
    name = "Windscar Whetstone",
    color = "acd2fd",
    meters = { spell = { "Slicing Maelstrom"} },
    display = "aggregate"
  },
  [142173] = { d = true,
    name = "Ring of Collapsing Futures",
    color = "7402f5",
    meters = { spell = { "Collapse" } }
  },
  [142174] = { d = true,
    name = "Choker of Barbed Reins",
    color = "555555",
    meters = { spell = { "Barbed Rebuke" } }
  },
  [133647] = {
    name = "Gift of Radiance",
    color = "feb806",
    meters = { spell = { "Spear of Light" } }
  },
  [133646] = { h = true,
    name = "Mote of Sanctification",
    color = "ff9b01",
    meters = { spell = { "Expel Light" } }
  },
  [137329] = {
    name = "Figurehead of the Naglfar",
    color = "c9c9c9",
    meters = { spell = { "Taint of the Sea" } }
  },
  [133645] = { h = true,
    name = "Naglfar Fare",
    color = "a0c95a",
    display = "aggregate"
  },
  [137357] = { d = true,
    name = "Mark of Dargrul",
    color = "fa380f",
    meters = { spell = { "Landslide" } }
  },
  [142159] = {
    name = "Bloodstained Handkerchief",
    color = "fa794f",
    meters = { spell = { "Cruel Garrote" } }
  },
  [142165] = { d = true,
    name = "Deteriorated Construct Core",
    color = "52fabb",
    display = "aggregate",
    meters = { spell = { "Volatile Energy" } }
  },
  [142158] = { h = true,
    name = "Faith's Crucible",
    color = "7a6927",
    meters = { spell = { "Holy Bolt" }}
  },
  [142160] = { d = true,
    name = "Mrrgria's Favor",
    color = "d7326a",
    meters = { spell = { "Thunder Ritual" } }
  },
  [142164] = { d = true,
    name = "Toe Knee's Promise",
    color = "fc2993",
    meters = { spell = { "Flame Gale" } }
  },
  [137419] = {
    name = "Chrono Shard",
    color = "fc09e2",
    type = "aura",
    meters = { aura = { "Acceleration" } }
  },
  [133766] = { h = true,
    name = "Nether Anti-Toxin",
    color = "fc2d62"
  },
  [137398] = {
    name = "Portable Manacracker",
    color = "38fcf9",
    meters = { spell = { "Withering Consumption" } }
  },
  [137406] = { d = true,
    name = "Terrorbound Nexus",
    color = "ce06fc",
    meters = { spell = { "Shadow Wave"} }
  }
}
makeAliases(Templates.item, "item", aliases, canonical)
Templates.aliases = aliases
Templates.canonical = canonical